# -*- coding: utf-8 -*-

__author__ = "Luis Barroso-Luque"
__copyright__ = "Copyright 2016, CAMERA, LBL, ALS"
__credits__ = ["Ronald J Pandolfi", "Dinesh Kumar", "Singanallur Venkatakrishnan", "Luis Luque", "Alexander Hexemer"]
__license__ = ""
__version__ = "1.2.1"
__maintainer__ = "Ronald J Pandolfi"
__email__ = "ronpandolfi@lbl.gov"
__status__ = "Beta"


import os
import tempfile
from functools import partial
from PySide import QtGui, QtCore
from collections import OrderedDict
from xicam import threads
from xicam import clientmanager as cmanager
from pipeline import pathtools, msg
from xicam import config
from modpkgs import guiinvoker

class LocalFileView(QtGui.QTreeView):
    """
    Local file explorer tree view
    """

    pathChanged = QtCore.Signal(str)
    sigOpen = QtCore.Signal(list)
    sigOpenFolder = QtCore.Signal(list)
    sigDelete = QtCore.Signal(list)
    sigUpload = QtCore.Signal(list)
    sigItemPreview = QtCore.Signal(str)

    def __init__(self, parent=None):
        super(LocalFileView, self).__init__(parent)

        self.file_model = QtGui.QFileSystemModel()
        self.setModel(self.file_model)
        self.path = config.settings['Default Local Path']
        if not self.path: self.path = os.path.expanduser('~')  # pathtools.getRoot()
        self.refresh(self.path)

        header = self.header()
        self.setHeaderHidden(True)
        for i in range(1, 4):
            header.hideSection(i)
#        filefilter = map(lambda s: '*'+s,loader.acceptableexts)
#        self.file_model.setNameFilters(filefilter)
        self.file_model.setNameFilterDisables(False)
        self.file_model.setResolveSymlinks(True)
        self.expandAll()
        self.sortByColumn(0, QtCore.Qt.AscendingOrder)
        self.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        self.setSelectionMode(self.ExtendedSelection)
        self.setIconSize(QtCore.QSize(16, 16))

        self.menu = QtGui.QMenu()
        standardActions = [QtGui.QAction('Open', self), QtGui.QAction('Open Folder', self),
                           QtGui.QAction('Delete', self)]
        standardActions[0].triggered.connect(self.handleOpenAction)
        standardActions[1].triggered.connect(self.handleOpenFolderAction)
        standardActions[2].triggered.connect(self.handleDeleteAction)
        self.menu.addActions(standardActions)

        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.menuRequested)

        self.doubleClicked.connect(self.onDoubleClick)

    def refresh(self, path=None):
        """
        Refresh the file tree, or switch directories and refresh
        """
        if path is None:
            path = self.path
        else:
            self.path = path

        if os.path.isdir(path):
            filter='*'
            root=path
        else:
            filter=os.path.basename(path)
            root=path[:-len(filter)]
        root = QtCore.QDir(root)
        self.file_model.setRootPath(root.absolutePath())
        self.file_model.setNameFilters([filter])
        self.setRootIndex(self.file_model.index(root.absolutePath()))
        self.pathChanged.emit(path)
        config.settings['Default Local Path'] = path

    def menuRequested(self, position):
        self.menu.exec_(self.viewport().mapToGlobal(position))

    def onDoubleClick(self, index):
        path = self.file_model.filePath(index)
        if os.path.isdir(path):
            self.refresh(path=path)
        else:
            self.sigOpen.emit([path])

    def currentChanged(self, current, previous):
        path = self.file_model.filePath(current)
        if os.path.isfile(path):
            self.sigItemPreview.emit(path)

    def getSelectedFilePaths(self):
        paths = [self.file_model.filePath(index) for index in self.selectedIndexes()]
        return paths

    def getSelectedFile(self):
        selected = str(self.file_model.fileName(self.currentIndex()))
        if selected == '':
            selected = None
        return selected

    def deleteSelection(self):
        for index in self.selectedIndexes():
            self.file_model.remove(index)

    def handleOpenAction(self):
        paths = self.getSelectedFilePaths()
        if os.path.isdir(paths[0]) and len(paths) == 1:
            self.refresh(path=paths[0])
        else:
            self.sigOpen.emit(paths)

    def handleOpenFolderAction(self):
        paths = self.getSelectedFilePaths()
        if os.path.isdir(paths[0]) and len(paths) == 1:
            self.sigOpenFolder.emit(paths)
        else:
            pass

    def handleDeleteAction(self):
        paths = self.getSelectedFilePaths()
        self.sigDelete.emit(paths)

    def handleUploadAction(self):
        paths = self.getSelectedFilePaths()
        self.sigUpload.emit(paths)


class RemoteFileView(QtGui.QListWidget):
    """
    Remote file explorer for REST API clients(NERSC and Globus)
    """
    pathChanged = QtCore.Signal(str)
    sigDelete = QtCore.Signal(list)
    sigOpen = QtCore.Signal(list)
    sigOpenFolder = QtCore.Signal(list)
    sigDownload = QtCore.Signal(str, str, object, tuple, dict, object)
    sigTransfer = QtCore.Signal(str, str, object, tuple, dict, object)
    sigItemPreview = QtCore.Signal(str)

    def __init__(self, remote_client, parent=None):
        super(RemoteFileView, self).__init__(parent)
        self.path = None
        self.client = remote_client
        self.itemDoubleClicked.connect(self.onDoubleClick)

        self.menu = QtGui.QMenu()
        standardActions = [QtGui.QAction('Open', self), QtGui.QAction('Open Folder', self). QtGui.QAction('Download', self),
                           QtGui.QAction('Delete', self), QtGui.QAction('Transfer', self)]
        standardActions[0].triggered.connect(self.handleOpenAction)
        standardActions[0].triggered.connect(self.handleOpenFolderAction)
        standardActions[2].triggered.connect(self.handleDownloadAction)
        standardActions[3].triggered.connect(self.handleDeleteAction)
        standardActions[4].triggered.connect(self.handleTransferAction)
        self.menu.addActions(standardActions)
        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.menuRequested)

    def refresh(self, path=None):
        if path is None:
            path = self.path
        else:
            self.path = path
        self.pathChanged.emit(path)

    def menuRequested(self, position):
        self.menu.exec_(self.viewport().mapToGlobal(position))

    def onDoubleClick(self, item):
        file_name = item.text(0)
        if len(file_name.split('.')) == 1:
            path = self.path + '/' + str(file_name)
            self.refresh(path=path)

    def currentChanged(self, current, previous):
        pass

    def getSelectedFilePaths(self):
        paths = ['{0}/{1}'.format(self.path, item.text()) for item in self.selectedItems()]
        return paths

    def getSelectedFile(self):
        return str(self.currentItem().text())

    @threads.method(callback_slot=lambda self: self.fillList)
    def getDirContents(self, path, *args):
        self.client.get_dir_contents(path, *args)

    def fillList(self, value):
        self.clear()
        for item in value:
            if item['name'] != '.' and item['name'] != '..':
                self.addItem(item['name'])

    def handleOpenAction(self):
        pass

    def handleOpenFolderAction(self):
        pass

    def handleTransferAction(self):
        pass

    def handleDownloadAction(self):
        pass

    def handleDeleteAction(self):
        pass


# This has been replaced with SFTP based client see SFTPFileView
# class NERSCFileView(RemoteFileView):
# """
#     File explorer for NERSC systems, must be passed a client and a worker to make REST calls
#     """
#
#     def __init__(self, nersc_client, system, parent=None):
#         self.system = system
#         super(NERSCFileView, self).__init__(nersc_client, parent=parent)
#         self.client.set_scratch_dir(system)
#         self.path = self.client.scratch_dir
#         self.getDirContents(self.path, self.system)
#
#     def refresh(self, path=None):
#         if path is None:
#             path = self.path
#
#         super(NERSCFileView, self).getDirContents(path, self.system)
#         super(NERSCFileView, self).refresh(path=path)
#
#    def onDoubleClick(self, item):
#        file_name = item.text()
#        if '.' in file_name:
#            save_path = os.path.join(tempfile.gettempdir(), file_name)
#            self.handleDownloadAction(save_path=save_path, fslot=(lambda: self.sigOpen.emit([save_path])))
#        super(NERSCFileView, self).onDoubleClick(item)
#
#     def handleOpenAction(self):
#         paths = self.getSelectedFilePaths()
#         files = [os.path.split(path)[-1] for path in paths]
#         for file_name in files:
#             save_path = os.path.join(tempfile.gettempdir(), file_name)
#             self.handleDownloadAction(save_path=save_path, fslot=(lambda: self.sigOpen.emit([save_path])))
#
#     def handleDownloadAction(self, save_path=None, fslot=None):
#         fpath = super(NERSCFileView, self).handleDownloadAction()
#         size = self.client.get_file_size(fpath, self.system)
#         desc = 'File {0} from {1}'.format(fpath, self.system)
#         if size < 100*2**20:
#             method = self.client.download_file_generator
#             args = [fpath, self.system]
#             kwargs = {}
#             if save_path is not None:
#                 kwargs['save_path'] = save_path
#         else:
#             # USE GLOBUS API CALL HERE, Think of how to get access to client
#             pass
#
#         self.sigDownload.emit(desc, method, args, kwargs, fslot)
#
#     def handleTransferAction(self):
#         return None
#
#     def handleDeleteAction(self):
#         runnable = threads.RunnableMethod(self.client.delete_file,
#                                           method_args=(self.getSelectedFilePath(), self.system),
#                                           finished_slot=self.refresh)
#         threads.add_to_queue(runnable)


class GlobusFileView(RemoteFileView):
    """
    File explorer for Globus endpoints
    """

    def __init__(self, endpoint, globus_client, parent=None):
        self.endpoint = endpoint
        super(GlobusFileView, self).__init__(globus_client, parent=parent)
        self.get_dir_contents('~')

    def refresh(self, path=None):
        if path is None:
            path = self.path
        super(GlobusFileView, self).getDirContents(path, self.endpoint)
        super(GlobusFileView, self).refresh(path=path)

    def deleteSelection(self):
        paths = self.getSelectedFilePaths()
        for path in paths:
            self.client.delete_file(self.endpoint, path)

    def handleDownloadAction(self, save_path=None, fslot=None):
        paths = super(GlobusFileView, self).getSelectedFilePaths()
        for path in paths:
            name = os.path.split(path)[-1]
            method = self.client.transfer_generator
            desc = '{0} from {1}.'.format(name, self.endpoint)
            args = [self.endpoint, path, 'LOCAL ENDPOINT']  # TODO Local endpoint again....
            kwargs = {}
            self.sigDownload.emit(name, desc, method, args, kwargs, fslot)

    def handleTransferAction(self):
        # TODO implement this
        paths = super(GlobusFileView, self).getSelectedFilePaths()
        for path in paths:
            method = self.client.transfer_generator
            desc = '{0} from {1}.'.format(os.path.split(path)[-1], self.endpoint)
            args = [self.endpoint, path, 'DEST ENDPOINT']  # TODO Dest endpoint geeet it....
            kwargs = {}
            return desc, method, args, kwargs

    def handleDeleteAction(self):
        paths = self.getSelectedFilePaths()
        self.sigDelete.emit(paths)


class SFTPFileView(QtGui.QTreeWidget):
    pathChanged = QtCore.Signal(str)
    sigAddTopLevelItem = QtCore.Signal(str, str)
    sigDelete = QtCore.Signal(list)
    sigOpen = QtCore.Signal(list)
    sigOpenFolder = QtCore.Signal(list)
    sigDownload = QtCore.Signal(str, str, object, tuple, dict, object)
    sigTransfer = QtCore.Signal(str, str, object, tuple, dict, object)
    sigItemPreview = QtCore.Signal(str)

    def __init__(self, sftp_client, parent=None):
        super(SFTPFileView, self).__init__(parent=parent)
        self.client = sftp_client
        self.path = sftp_client.pwd

        self.sigAddTopLevelItem.connect(self.addTopLevelItem)

        self.itemExpanded.connect(self.getItemChildren)
        self.itemDoubleClicked.connect(self.onDoubleClick)
        self.setHeaderHidden(True)
        self.refresh()

        self.menu = QtGui.QMenu(parent=self)
        openAction = QtGui.QAction('Open', self)
        openFolderAction = QtGui.QAction('Open Folder', self)
        downloadAction = QtGui.QAction('Download', self)
        deleteAction = QtGui.QAction('Delete', self)
        openAction.triggered.connect(self.handleOpenAction)
        downloadAction.triggered.connect(self.handleDownloadAction)
        deleteAction.triggered.connect(self.handleDeleteAction)
        self.menu.addActions([openAction, openFolderAction, downloadAction, deleteAction])

        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.menuRequested)

    @threads.method(callback_slot=lambda self: self.pathChanged.emit(self.path))
    def refresh(self, path=None):
        guiinvoker.invoke_in_main_thread(self.clear)
        if path is not None:
            self.path = path
            self.client.cd(path)
        self.client.walktree(self.path, lambda x: self.sigAddTopLevelItem.emit(x, 'file'),
                             lambda x: self.sigAddTopLevelItem.emit(x, 'dir'), lambda: None, recurse=False)
        return self
        # runnable = threads.RunnableMethod(self.client.walktree,
        #                                   method_args=(self.path,
        #                                                lambda x: self.sigAddTopLevelItem.emit(x, 'file'),
        #                                                lambda x: self.sigAddTopLevelItem.emit(x, 'dir'),
        #                                                lambda: None),
        #                                   method_kwargs={'recurse': False},
        #                                   finished_slot=lambda: self.pathChanged.emit(self.path))
        # threads.add_to_queue(runnable)
        # Or on gui thread
        # self.client.walktree(self.path,
        # lambda x: self.createTopLevelItem(x, 'file'),
        #                      lambda x: self.createTopLevelItem(x, 'dir'),
        #                      lambda x: x, recurse=False)
        # self.pathChanged.emit(self.path)

    def menuRequested(self, position):
        self.menu.exec_(self.viewport().mapToGlobal(position))

    def addTopLevelItem(self, path, type):
        name = os.path.split(path)[-1]
        if type == 'file':
            icon = QtGui.QFileIconProvider().icon(QtGui.QFileIconProvider.File)
            item = SFTPFileTreeItem(name, path, icon, self)
        elif type == 'dir':
            name = os.path.split(path)[-1]
            icon = QtGui.QFileIconProvider().icon(QtGui.QFileIconProvider.Folder)
            item = SFTPDirTreeItem(name, self.client, path, icon, self)
        else:
            return
        super(SFTPFileView, self).addTopLevelItem(item)

    @staticmethod
    def isDir(item):
        if isinstance(item, SFTPDirTreeItem):
            return True
        else:
            return False

    def onDoubleClick(self, item):
        if self.isDir(item):
            self.path = item.path
            self.refresh()
        else:
            file_name = item.text(0)
            save_path = [os.path.join(tempfile.gettempdir(), file_name)]
            self.handleDownloadAction(save_paths=save_path, fslot=(lambda: self.sigOpen.emit(save_path)))

    def getItemChildren(self, item):
        if item.childCount() == 0:
            item.getChildren()

    def currentChanged(self, current, previous):
        pass

    def getSelectedFilePaths(self):
        paths = [item.path for item in self.selectedItems()]
        return paths

    def deleteSelection(self):
        paths = self.getSelectedFilePaths()
        for path in paths:
            self.client.remove(path)
        self.refresh()

    def handleOpenAction(self):
        paths = self.getSelectedFilePaths()
        save_paths = [os.path.join(tempfile.gettempdir(), os.path.split(path)[-1]) for path in paths]
        for save_path in save_paths:
            self.handleDownloadAction(save_paths=[save_path], fslot=(lambda: self.sigOpen.emit([save_path])))

    def handleDownloadAction(self, save_paths=None, fslot=None):
        paths = self.getSelectedFilePaths()
        items = self.selectedItems()
        if save_paths is None:
            save_paths = len(paths) * (None,)
        for path, item, save_path in zip(paths, items, save_paths):
            name = os.path.split(path)[-1]
            desc = '{0} from {1}'.format(name, self.client.host)
            args = (path,)
            kwargs = {}
            if save_path is not None:
                kwargs['localpath'] = save_path
            if self.isDir(item):
                method = self.client.get_r
            else:
                method = self.client.get
            self.sigDownload.emit(name, desc, method, args, kwargs, fslot)

    def handleDeleteAction(self):
        paths = self.getSelectedFilePaths()
        self.sigDelete.emit(paths)


class SpotDatasetView(QtGui.QTreeWidget):
    """
    Tree widgets showing Spot datasets as a file directory
    """

    sigOpen = QtCore.Signal(list)
    sigOpenFolder = QtCore.Signal(list)
    sigDownload = QtCore.Signal(str, str, object, tuple, dict, object)
    sigTransfer = QtCore.Signal(str, str, object, tuple, dict, object)
    sigItemPreview = QtCore.Signal(object)


    def __init__(self, spot_client, parent=None):
        super(SpotDatasetView, self).__init__(parent)
        self.client = spot_client
        self.search_params = {'skipnum': '0', 'sortterm': 'fs.stage_date', 'sortterm': 'appmetadata.sdate',
                              'sorttype': 'desc', 'end_station': 'bl832'}
        self.getDatasets('')
        self.setHeaderHidden(True)

        self.menu = QtGui.QMenu()
        standardActions = [QtGui.QAction('Open', self), QtGui.QAction('Download', self)]
        #, QtGui.QAction('Preview', self), QtGui.QAction('Transfer', self)]
        standardActions[0].triggered.connect(self.handleOpenAction)
        standardActions[1].triggered.connect(self.handleDownloadAction)
        # standardActions[2].triggered.connect(self.handlePreviewAction)
        # standardActions[3].triggered.connect(self.handleTransferAction)
        self.menu.addActions(standardActions)
        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.menuRequested)
        self.itemDoubleClicked.connect(self.onDoubleClick)

    def menuRequested(self, position):
        if self.currentItem().childCount() != 0:
            return
        self.menu.exec_(self.viewport().mapToGlobal(position))

    def onDoubleClick(self, item):
        if item.childCount() != 0:
            return
        file_name = item.text(0)
        save_path = [os.path.join(tempfile.gettempdir(), file_name)]
        self.handleDownloadAction(save_paths=save_path, fslot=(lambda: self.sigOpen.emit(save_path)))

    def getDatasets(self, query):
        msg.showMessage('Searching SPOT database...')
        search = threads.method(callback_slot=self.createDatasetDictionary,
                                finished_slot=msg.clearMessage)(self.client.search)
        search(query, **self.search_params)

    @QtCore.Slot(dict)
    def createDatasetDictionary(self, data):
        tree_data = {}
        for index in range(len(data)):
            derived_data = {data[index]['fs']['stage']:
                            data[index]['name']}
            if 'derivatives' in data[index]['fs'].keys():
                derivatives = data[index]['fs']['derivatives']
                for d_index in range(len(derivatives)):
                    stage = derivatives[d_index]['dstage']
                    name = derivatives[d_index]['did'].split('/')[-1]
                    derived_data[stage] = name

            dataset = data[index]['fs']['dataset']
            tree_data[dataset] = derived_data
            self.fillTree(tree_data)

    def fillTree(self, data):
        self.clear()
        item = self.invisibleRootItem()
        self.addTreeItems(item, data)

    def addTreeItems(self, item, value):
        item.setExpanded(False)
        if type(value) is dict:
            for key, val in sorted(value.iteritems()):
                icon = QtGui.QFileIconProvider().icon(QtGui.QFileIconProvider.Folder)
                child = QtGui.QTreeWidgetItem([key], parent=self)
                child.setIcon(0, icon)
                item.addChild(child)
                self.addTreeItems(child, val)
        else:
            icon = QtGui.QFileIconProvider().icon(QtGui.QFileIconProvider.File)
            child = QtGui.QTreeWidgetItem([value], parent=self)
            child.setIcon(0, icon)
            item.addChild(child)

    def currentChanged(self, current, previous):
        item = self.itemFromIndex(current)
        try:
            if item.childCount() == 0:
                msg.showMessage('Loading preview...')
                dataset = item.parent().parent().text(0)
                stage = item.parent().text(0)
                bg_get_preview = threads.method(callback_slot=self.sigItemPreview.emit,
                                                except_slot=self.handleException)(self.client.get_image_as)
                bg_get_preview(dataset, stage, index=0)
        except AttributeError:
            pass

    def getStagesAndDatasets(self):
        dsets_stages = [(item.parent().parent().text(0), item.parent().text(0))
                        for item in self.selectedItems() if item.childCount() == 0]
        return dsets_stages

    def getSelectedDatasets(self):
        datasets = [item.text(0) for item in self.selectedItems() if item.childCount() == 0]
        return datasets

    def handleOpenAction(self):
        save_paths = map(lambda dset: os.path.join(tempfile.gettempdir(), dset), self.getSelectedDatasets())
        for path in save_paths:
            self.handleDownloadAction(save_paths=[path], fslot=(lambda: self.sigOpen.emit([path])))

    def handleDownloadAction(self, save_paths=None, fslot=None):
        names = self.getSelectedDatasets()
        stages_dsets = self.getStagesAndDatasets()
        if save_paths is None:
            save_paths = len(names) * (None, )
        for name, stage_dset, save_path in zip(names, stages_dsets, save_paths):
            desc = '{} from SPOT.'.format(name)
            method = self.client.download_dataset_generator
            args = stage_dset
            kwargs = {}
            if save_path is not None:
                kwargs['save_path'] = save_path
            self.sigDownload.emit(name, desc, method, args, kwargs, fslot)

    def handleTransferAction(self):
        # TODO need to implement this
        fname = self.getSelectedDatasets()
        stage, dataset = self.getStagesAndDatasets()
        system = self.client.system
        path = self.client.scratch_dir
        desc = '{0} transfer from spot to {1}.'.format(fname, system)
        method = self.client.transfer_2_nersc
        args = [dataset, stage, path, system]
        kwargs = {}
        return desc, method, args, kwargs

    def handleException(self, ex, tb):
        msg.showMessage('Unable to fetch preview from SPOT.')
        msg.logMessage(ex, level=40)


class FileExplorer(QtGui.QWidget):
    """
    Container for local and remote file tree view that adds back button and path label
    """

    def __init__(self, file_view, parent=None):
        super(FileExplorer, self).__init__(parent)

        self.file_view = file_view
        self.path_label = QtGui.QLineEdit(self)
        self.back_button = QtGui.QToolButton(self)
        self.back_button.setToolTip('Back')
        self.refresh_button = QtGui.QToolButton(self)
        self.refresh_button.setToolTip('Refresh')

        for button, icon_file in zip((self.back_button, self.refresh_button),
                                     ('xicam/gui/icons_44.png', 'xicam/gui/icons_57.png')):
            icon = QtGui.QIcon()
            icon.addPixmap(QtGui.QPixmap(icon_file), QtGui.QIcon.Normal, QtGui.QIcon.Off)
            button.setIcon(icon)
            button.setIconSize(QtCore.QSize(18, 18))
            button.setFixedSize(32, 32)

        # self.path_label.setReadOnly(True)

        l = QtGui.QVBoxLayout(self)
        l.setStretch(0, 0)
        l.setContentsMargins(0, 0, 0, 0)
        l.setSpacing(0)
        hl = QtGui.QHBoxLayout()
        hl.addWidget(self.back_button)
        hl.addWidget(self.refresh_button)
        hl.addWidget(self.path_label)
        hl.setStretch(0, 0)
        l.addLayout(hl)
        l.addWidget(self.file_view)

        self.setPathLabel(self.file_view.path)
        self.back_button.clicked.connect(self.onBackClicked)
        self.refresh_button.clicked.connect(self.onRefreshClicked)
        self.file_view.pathChanged.connect(self.setPathLabel)
        self.path_label.textChanged.connect(self.pathlabelChanged)

    def pathlabelChanged(self):
        path = self.path_label.text()
        self.file_view.refresh(path=path)

    def onBackClicked(self):
        path = self.file_view.path
        path = os.path.dirname(str(path))
        self.file_view.refresh(path=path)

    def onRefreshClicked(self):
        self.file_view.refresh()

    def setPathLabel(self, path):
        self.path_label.setText(path)

    def getSelectedFilePaths(self):
        return self.file_view.getSelectedFilePaths()


class SpotDatasetExplorer(QtGui.QWidget):
    """
    Class that holds a SpotDatasetView and additional search boxes
    """

    def __init__(self, spot_client, parent=None):
        super(SpotDatasetExplorer, self).__init__(parent)

        self.file_view = SpotDatasetView(spot_client, self)
        self.search_box = QtGui.QLineEdit(self)
        self.sort_box = QtGui.QComboBox(self)
        self.order_box = QtGui.QComboBox(self)

        self.search_box.setPlaceholderText('Search Term')
        self.sort_box.addItems(['Beamline date', 'Processed date'])
        self.order_box.addItems(['Descending', 'Ascending'])

        l = QtGui.QVBoxLayout(self)
        l.setStretch(0, 0)
        l.setContentsMargins(0, 0, 0, 0)
        l.setSpacing(0)
        hl = QtGui.QHBoxLayout()
        hl.addWidget(self.sort_box)
        hl.addWidget(self.order_box)
        vl = QtGui.QVBoxLayout()
        vl.addWidget(self.search_box)
        vl.addLayout(hl)
        l.addLayout(vl)
        l.addWidget(self.file_view)

        self.search_box.returnPressed.connect(self.searchReturnPressed)

    def searchReturnPressed(self):
        if self.sort_box.currentText() == 'Beamline date':
            self.file_view.search_params['sortterm'] = 'appmetadata.sdate'
        else:
            self.file_view.search_params['sortterm'] = 'appmetadata.stage_date'

        if self.order_box.currentText() == 'Descending':
            self.file_view.search_params['sorttype'] = 'desc'
        else:
            self.file_view.search_params['sorttype'] = 'asc'
        query = str(self.search_box.text())

        self.file_view.getDatasets(query)

    def getSelectedFilePath(self):
        return os.path.join(*self.file_view.getStagesAndDatasets())

    def getRawDatasetList(self):
        widget = self.file_view
        items = []

        parent_items = widget.findItems('*', QtCore.Qt.MatchWildcard)
        child_items = []
        for item in parent_items:
            child_items += [item.child(i) for i in range(item.childCount())]
        for item in child_items:
            items += [item.child(i).text(0) for i in range(item.childCount()) if item.text(0) == 'raw']
        return items


class MultipleFileExplorer(QtGui.QTabWidget):
    """
    Class for multiple location file explorer, capability to add GlobusFileView Tabs
    """

    sigLoginSuccess = QtCore.Signal(bool)
    sigLoginRequest = QtCore.Signal(QtCore.Signal, bool)
    sigProgJob = QtCore.Signal(str, object, list, dict, object)
    sigPulsJob = QtCore.Signal(str, object, list, dict, object)
    sigSFTPJob = QtCore.Signal(str, object, list, dict, object)
    sigOpen = QtCore.Signal(list)
    sigFolderOpen = QtCore.Signal(list)
    sigPreview = QtCore.Signal(object)

    def __init__(self, parent=None):
        super(MultipleFileExplorer, self).__init__(parent)
        self.explorers = OrderedDict()

        self.tab = TabBarPlus()
        self.setTabBar(self.tab)
        self.setTabsClosable(True)

        self.explorers['Local'] = FileExplorer(LocalFileView(self), self)
        self.addFileExplorer('Local', self.explorers['Local'], closable=False)

        self.jobtab = JobTable(self)
        # Do not understand why I need to add it and remove it so that its not added as a seperate widget
        self.addTab(self.jobtab, 'Jobs')
        self.removeTab(1)

        self.sigProgJob.connect(self.jobtab.addProgJob)
        self.sigPulsJob.connect(self.jobtab.addPulseJob)
        self.sigSFTPJob.connect(self.jobtab.addSFTPJob)

        self.tab.plusClicked.connect(self.onPlusClicked)
        self.tabCloseRequested.connect(self.removeTab)

        self.newtabmenu = QtGui.QMenu(None)
        addspot = QtGui.QAction('SPOT', self.newtabmenu)
        addcori = QtGui.QAction('Cori', self.newtabmenu)
        addedison = QtGui.QAction('Edison', self.newtabmenu)
        addbragg = QtGui.QAction('Bragg', self.newtabmenu)
        addsftp = QtGui.QAction('SFTP Connection', self.newtabmenu)
        showjobtab = QtGui.QAction('Jobs', self.newtabmenu)
        self.standard_actions = OrderedDict({'SPOT': addspot, 'Cori': addcori, 'Edison': addedison,
                                             'Bragg': addbragg, 'SFTP': addsftp})
        self.newtabmenu.addActions(self.standard_actions.values())
        self.newtabmenu.addAction(showjobtab)
        addspot.triggered.connect(self.addSPOTTab)
        addedison.triggered.connect(lambda: self.addHPCTab('Edison'))
        addcori.triggered.connect(lambda: self.addHPCTab('Cori'))
        addbragg.triggered.connect(lambda: self.addHPCTab('Bragg'))
        addsftp.triggered.connect(self.addSFTPTab)
        showjobtab.triggered.connect(lambda: self.addTab(self.jobtab, 'Jobs'))

    def enableActions(self):
        for name, action in self.standard_actions.iteritems():
            if name in self.explorers.keys():
                action.setEnabled(False)
            else:
                action.setEnabled(True)

    def removeTab(self, p_int):
        if self.tabText(p_int) != 'Jobs':
            name = self.explorers.keys()[p_int]
            explorer = self.explorers.pop(name)
            cmanager.logout(explorer.file_view.client)
            self.widget(p_int).deleteLater()
            self.enableActions()
        super(MultipleFileExplorer, self).removeTab(p_int)

    def removeTabs(self):
        for i in xrange(1, self.count()):
            self.removeTab(1)

    def onPlusClicked(self):
        self.newtabmenu.popup(QtGui.QCursor.pos())

    def addFileExplorer(self, name, file_explorer, closable=True):
        self.explorers[name] = file_explorer
        file_explorer.file_view.sigItemPreview.connect(self.itemSelected)
        self.wireExplorerSignals(file_explorer)
        idx = len(self.explorers) - 1
        tab = self.insertTab(idx, file_explorer, name)
        if closable is False:
            try:
                self.tabBar().tabButton(tab, QtGui.QTabBar.RightSide).resize(0, 0)
                self.tabBar().tabButton(tab, QtGui.QTabBar.RightSide).hide()
            except AttributeError:
                self.tabBar().tabButton(tab, QtGui.QTabBar.LeftSide).resize(0, 0)
                self.tabBar().tabButton(tab, QtGui.QTabBar.LeftSide).hide()
        self.setCurrentWidget(file_explorer)

    def wireExplorerSignals(self, explorer):
        explorer.file_view.sigOpen.connect(self.handleOpenActions)
        explorer.file_view.sigOpenFolder.connect(self.handleOpenFolderActions)
        try:
            explorer.file_view.sigDownload.connect(self.handleDownloadActions)
        except AttributeError:
            pass
        try:
            explorer.file_view.sigDelete.connect(self.handleDeleteActions)
        except AttributeError:
            pass
        try:
            explorer.file_view.sigTransfer.connect(self.handleTransferActions)
        except AttributeError:
            pass

    def itemSelected(self, item):
        msg.clearMessage() #
        self.sigPreview.emit(item)

    def addHPCTab(self, system):
        # # NERSC tabs based on NEWT API
        # add_nersc_explorer = lambda client: self.addFileExplorer(system.capitalize(),
        # FileExplorer(NERSCFileView(client, system, self)))
        # login_callback = lambda client: self.loginSuccess(client, add_explorer=add_nersc_explorer)
        # self.sigLoginRequest.emit(partial(cmanager.login, login_callback, cmanager.spot_client.login), False)
        # add_sftp_explorer = lambda client: self.addFileExplorer(client.host.split('.')[0],
        #                                                         FileExplorer(SFTPTreeWidget(client, self)))

        # NERSC tabs based on SFTP
        add_sftp_explorer = lambda client: self.addFileExplorer(system,
                                                                FileExplorer(SFTPFileView(client, self)))
        add_sftp_callback = lambda client: self.loginSuccess(client,
                                                             add_explorer=add_sftp_explorer)
        login_callback = lambda client: cmanager.add_sftp_client(system,
                                                                 client,
                                                                 add_sftp_callback)
        sftp_client = partial(cmanager.sftp_client, cmanager.HPC_SYSTEM_ADDRESSES[system])
        self.sigLoginRequest.emit(partial(cmanager.login, login_callback, sftp_client), False)

    def addSPOTTab(self):
        add_spot_explorer = lambda client: self.addFileExplorer('SPOT',
                                                                SpotDatasetExplorer(client, self))
        login_callback = lambda client: self.loginSuccess(client,
                                                          add_explorer=add_spot_explorer)
        self.sigLoginRequest.emit(partial(cmanager.login, login_callback, cmanager.spot_client.login), False)

    #TODO add globus tranfer capabilities to SFTP connected machines if they are globus endpoints
    # This is being replaced by SFTP tabs
    # def addGlobusTab(self, endpoint):
    # add_globus_explorer = lambda client: self.addFileExplorer(endpoint.split('#')[-1],
    #                                                              FileExplorer(GlobusFileView(client, client, self)))
    #     add_endpoint_callback = lambda client: self.loginSuccess(client,
    #                                                              add_explorer=add_globus_explorer)
    #     login_callback = lambda client: cmanager.add_globus_client(endpoint.split('#')[-1],
    #                                                                client,
    #                                                                add_endpoint_callback)
    #     globus_client = cmanager.globus_client()
    #     self.sigLoginRequest.emit(partial(cmanager.login, login_callback, globus_client.login), False)

    def addSFTPTab(self):
        add_sftp_explorer = lambda client: self.addFileExplorer(client.host.split('.')[0],
                                                                FileExplorer(SFTPFileView(client, self)))
        add_sftp_callback = lambda client: self.loginSuccess(client,
                                                             add_explorer=add_sftp_explorer)
        login_callback = lambda client: cmanager.add_sftp_client(client.host,
                                                                 client,
                                                                 add_sftp_callback)
        sftp_client = cmanager.sftp_client
        self.sigLoginRequest.emit(partial(cmanager.login, login_callback, sftp_client), True)

    def loginSuccess(self, client, add_explorer=None):
        if not client:
            self.sigLoginSuccess.emit(False)
        else:
            add_explorer(client)
            self.enableActions()
            self.sigLoginSuccess.emit(True)

    def getSelectedFilePaths(self):
        return self.currentWidget().getSelectedFilePaths()

    def getCurrentPath(self):
        return self.currentWidget().path

    def getPath(self, tab_name):
        return self.explorers[tab_name].path

    def handleOpenActions(self, paths):
        if len(paths) > 0:
            self.sigOpen.emit(paths)

    def handleOpenFolderActions(self, paths):
        if len(paths) > 0:
            self.sigFolderOpen.emit(paths)

    def handleDeleteActions(self, paths):
        r = QtGui.QMessageBox.warning(self, 'Delete file',
                                      'Are you sure you want to delete\n{}?'.format(',\n'.join(paths)),
                                      QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        if r == QtGui.QMessageBox.Yes:
            self.currentWidget().file_view.deleteSelection()

    def handleUploadAction(self, desc, method, args, kwargs):
        self.sigProgJob.emit(desc, method, args, kwargs)
        self.addTab(self.jobtab, 'Jobs')

    def handleDownloadActions(self, name, desc, method, args, kwargs, fslot):
        if 'save_path' not in kwargs and 'localpath' not in kwargs:
            fileDialog = QtGui.QFileDialog(self, 'Save as', os.path.expanduser('~'))
            fileDialog.setAcceptMode(QtGui.QFileDialog.AcceptSave)
            fileDialog.selectFile(name)
            if fileDialog.exec_():
                save_path = str(fileDialog.selectedFiles()[0])
                if isinstance(self.currentWidget().file_view, SFTPFileView):
                    kwargs['localpath'] = save_path
                else:
                    kwargs['save_path'] = save_path
        if isinstance(self.currentWidget().file_view, SFTPFileView):
            self.sigSFTPJob.emit(desc, method, args, kwargs, fslot)
        else:
            self.sigProgJob.emit(desc, method, args, kwargs, fslot)
        self.addTab(self.jobtab, 'Jobs')

    def handleTransferActions(self, paths, desc, method, args, kwargs):
        #TODO Need to implement this
        if isinstance(self.currentWidget().file_view, SpotDatasetView):
            self.sigPulsJob.emit(desc, method, args, kwargs)
        else:
            self.sigProgJob.emit(desc, method, args, kwargs)
        self.addTab(self.jobtab, 'Jobs')


class TabBarPlus(QtGui.QTabBar):
    """
    Tab bar that has a plus button floating to the right of the tabs.
    """

    plusClicked = QtCore.Signal()

    def __init__(self, parent=None):
        super(TabBarPlus, self).__init__(parent)

        self.plus_button = QtGui.QPushButton(" + ")
        self.plus_button.setToolTip('Add browser location')
        self.plus_button.setParent(self)
        self.plus_button.setMaximumSize(32, 32)
        self.plus_button.setMinimumSize(32, 32)
        self.plus_button.clicked.connect(self.plusClicked.emit)
        self.movePlusButton()  # Move to the correct location
        self.setDocumentMode(True)

    def sizeHint(self):
        sizeHint = QtGui.QTabBar.sizeHint(self)
        width = sizeHint.width()
        height = sizeHint.height()
        return QtCore.QSize(width + 32, height)

    def resizeEvent(self, event):
        super(TabBarPlus, self).resizeEvent(event)
        self.movePlusButton()

    def tabLayoutChange(self):
        super(TabBarPlus, self).tabLayoutChange()
        self.movePlusButton()

    def movePlusButton(self):
        # Find the width of all of the tabs
        size = 0
        for i in range(self.count()):
            size += self.tabRect(i).width()

        h = self.geometry().top()
        w = self.width()
        if size > w:
            self.plus_button.move(w, h)
        else:
            self.plus_button.move(size, h)


class JobTable(QtGui.QTableWidget):
    """
    Class with table of download, upload and transfer jobs entries.
    """

    def __init__(self, parent=None):
        super(JobTable, self).__init__(0, 2, parent=parent)
        self.jobs = []

        self.setHorizontalHeaderLabels(['Progress', 'Description'])
        self.setFrameShape(QtGui.QFrame.NoFrame)
        self.verticalHeader().hide()
        self.horizontalHeader().setClickable(False)
        self.horizontalHeader().setStretchLastSection(True)
        self.horizontalHeader().setDefaultAlignment(QtCore.Qt.AlignLeft)
        self.setShowGrid(False)

        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.menu = QtGui.QMenu(self)
        cancel = QtGui.QAction('Cancel', self)
        cancel.triggered.connect(self.cancelActionTriggered)
        remove = QtGui.QAction('Remove', self)
        remove.triggered.connect(self.removeActionTriggered)
        self.menu.addActions([cancel, remove])
        self.customContextMenuRequested.connect(self.menuRequested)

    def menuRequested(self, pos):
        self.menu.exec_(self.mapToGlobal(pos))

    def cancelActionTriggered(self):
        self.jobs[self.currentRow()].cancel()

    def removeActionTriggered(self):
        self.removeJob(self.jobs[self.currentRow()])

    def addJob(self, job_desc):
        row_num = self.rowCount()
        self.insertRow(row_num)
        jobentry = JobEntry()
        jobentry.setDescription(job_desc)
        self.jobs.append(jobentry)
        self.setCellWidget(row_num, 0, jobentry.widget)
        self.setCellWidget(row_num, 1, jobentry.desc_label)
        return jobentry

    @QtCore.Slot(str, object, list, dict)
    def addProgJob(self, job_desc, generator, args, kwargs, finish_slot=None):
        job_entry = self.addJob(job_desc)
        bg_generator = threads.iterator(callback_slot=job_entry.progress,
                                        finished_slot=finish_slot,
                                        interrupt_signal=job_entry.sigCancel)(generator)
        bg_generator(*args, **kwargs)

    def addSFTPJob(self, job_desc, method, args, kwargs, finish_slot=None):
        job_entry = self.addJob(job_desc)
        kwargs['callback'] = job_entry.progressRaw
        bg_method = threads.method(finished_slot=finish_slot)(method)
        bg_method(*args, **kwargs)

    @QtCore.Slot(str, object, list, dict)
    def addPulseJob(self, job_type, job_desc, method, args, kwargs):
        job_entry = self.addJob(job_type, job_desc)
        job_entry.sigRemove.connect(self.removeJob)
        job_entry.pulseStart()
        bg_method = threads.method(finished_slot=job_entry.pulseStop, interrupt_signal=job_entry.sigCancel)(method)
        bg_method(*args, **kwargs)


    def removeJob(self, jobentry):
        jobentry.cancel()
        idx = self.jobs.index(jobentry)
        del self.jobs[idx]
        self.removeRow(idx)
        jobentry.deleteLater()


class JobEntry(QtGui.QWidget):
    """
    Class for job entries (downloads/uploads/transfers) in Job Table. Each job entry has a description and a progress
    bar to show fractional prograss or pulsing.
    """

    sigCancel = QtCore.Signal()
    sigRemove = QtCore.Signal(object)

    def __init__(self):
        super(JobEntry, self).__init__()
        self.desc_label = QtGui.QLabel()
        self.desc_label.setAlignment(QtCore.Qt.AlignVCenter)
        self.progressbar = QtGui.QProgressBar()
        self.progressbar.setRange(0, 100)
        self.widget = QtGui.QWidget()
        self.widget.setLayout(QtGui.QHBoxLayout())
        self.widget.layout().setContentsMargins(0, 0, 0, 0)
        self.widget.layout().addWidget(self.progressbar)

    def setDescription(self, desc):
        self.desc_label.setText(desc)

    def cancel(self):
        self.sigCancel.emit()

    def progress(self, i):
        i = int(i*100)
        self.progressbar.setValue(i)

    def progressRaw(self, transfered, total):
        self.progress(transfered/total)

    def pulseStart(self):
        self.progressbar.setRange(0, 0)

    def pulseStop(self):
        self.progressbar.setRange(0, 100)
        self.progress(1)


class LazyTreeItem(QtGui.QTreeWidgetItem):
    """
    Base class for a lazy tree item that does not get its children until asked to
    """

    def __init__(self, name, icon=None, parent=None):
        super(LazyTreeItem, self).__init__([name], parent=parent)
        self.parentItem = parent
        if icon is not None:
            self.setIcon(0, icon)
        self.setChildIndicatorPolicy(QtGui.QTreeWidgetItem.ShowIndicator)

    def setExpanded(self, expand):
        if expand and self.childCount() == 0:
            self.getChildren()
        super(LazyTreeItem, self).setExpanded(expand)

    def getChildren(self):
        # Override this function to get children
        raise NotImplementedError('getChildren method not implemented. This method must be implemented.')


# TODO walktree in a different thread!
class SFTPDirTreeItem(LazyTreeItem, QtCore.QObject):
    """
    SFTP folder tree item that will not retrieve its contents until it is expanded
    """
    sigAddChildFile = QtCore.Signal(str)
    sigAddChildDir = QtCore.Signal(str)

    def __init__(self, name, client, path, icon=None, parent=None):
        # super(SFTPDirTreeItem, self).__init__(name, icon, parent)
        LazyTreeItem.__init__(self, name, icon, parent)
        QtCore.QObject.__init__(self, parent=parent)
        self.client = client
        self.path = path
        self.sigAddChildFile.connect(self.addChildFile)
        self.sigAddChildDir.connect(self.addChildDir)

    @threads.method()
    def getChildren(self):
        self.client.walktree(self.path, self.sigAddChildFile.emit, self.sigAddChildDir.emit,
                             self.handleUnknown, recurse=False)

    def addChildFile(self, path):
        name = os.path.split(path)[-1]
        icon = QtGui.QFileIconProvider().icon(QtGui.QFileIconProvider.File)
        item = SFTPFileTreeItem(name, path, icon=icon, parent=self)
        item.setIcon(0, icon)
        self.addChild(item)


    def addChildDir(self, path):
        name = os.path.split(path)[-1]
        icon = QtGui.QFileIconProvider().icon(QtGui.QFileIconProvider.Folder)
        item = SFTPDirTreeItem(name, self.client, path, icon, self)
        item.setIcon(0, icon)
        self.addChild(item)

    def handleUnknown(self, path):
        msg.logMessage('Unknown object found: {0}'.format(path),msg.WARNING)


class SFTPFileTreeItem(QtGui.QTreeWidgetItem):
    """
    SFTP File tree item that saves its path
    """

    def __init__(self, name, path, icon=None, parent=None):
        super(SFTPFileTreeItem, self).__init__([name], parent=parent)
        self.parentItem = parent
        if icon is not None:
            self.setIcon(0, icon)
        self.path = path


if __name__ == '__main__':
    import client
    import getpass
    import sys

    passw = getpass.getpass()
    client = client.sftp.SFTPClient('bl832viz2.dhcp.lbl.gov', username='lbluque', password=passw)
    app = QtGui.QApplication(sys.argv)
    w = SFTPFileView(client)
    w.setWindowTitle("Test this thing")
    w.show()
    sys.exit(app.exec_())


import pyqtgraph as pg
import pyqtgraph.opengl as gl
from OpenGL.GL import *
import latvec
import numpy as np
from PySide import QtGui
import featuremanager
import customwidgets

viewWidget = None

boxsize = 100

def load():
    global viewWidget
    viewWidget = orthoGLViewWidget()
    redraw()


class orthoGLViewWidget(gl.GLViewWidget):
    def __init__(self):
        super(orthoGLViewWidget, self).__init__()
        self.opts['distance']=1500

    def projectionMatrix(self, region=None):
        # Xw = (Xnd + 1) * width/2 + X
        if region is None:
            region = (0, 0, self.width(), self.height())

        x0, y0, w, h = self.getViewport()
        dist = self.opts['distance']
        fov = self.opts['fov']
        nearClip = dist * 0.0001
        farClip = dist * 1000.

        r = nearClip * np.tan(fov * 0.5 * np.pi / 180.)
        t = r * h / w

        # convert screen coordinates (region) to normalized device coordinates
        # Xnd = (Xw - X0) * 2/width - 1
        ## Note that X0 and width in these equations must be the values used in viewport
        left = r * ((region[0] - x0) * (2.0 / w) - 1)
        right = r * ((region[0] + region[2] - x0) * (2.0 / w) - 1)
        bottom = t * ((region[1] - y0) * (2.0 / h) - 1)
        top = t * ((region[1] + region[3] - y0) * (2.0 / h) - 1)

        tr = QtGui.QMatrix4x4()
        tr.ortho(left * 2000, right * 2000, bottom * 2000, top * 2000, -10, farClip)
        return tr


class latticeFrame(gl.GLGridItem):
    def __init__(self, lines, antialias=True, glOptions='translucent'):
        super(latticeFrame, self).__init__(size=None, color=None)#, antialias=True, glOptions='translucent')
        self.setGLOptions(glOptions)
        self.antialias = antialias
        self.lines = lines
       # print lines


    def paint(self):
        # print 'painting!'
        self.setupGLState()
        if self.antialias:
            glEnable(GL_LINE_SMOOTH)
            glEnable(GL_BLEND)
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
            glHint(GL_LINE_SMOOTH_HINT, GL_NICEST)

        # glEnable(GL_LINE_SMOOTH)
        # glEnable(GL_BLEND)
        # glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        # glHint(GL_LINE_SMOOTH_HINT, GL_NICEST)
        #
        glLineWidth(3)
        #glColor4f(1, .5, 0., .65)
        glColor4f(0,1.,1.,.65)

        glBegin(GL_LINES)
        for line in self.lines:
            # print 'line:',line
            for vertex in line:
                glVertex3fv(vertex)
                # print 'vertex:', vertex
        glEnd()
        glLineWidth(1)


def clear():

    for item in viewWidget.items:
        item._setView(None)
    grid = gl.GLGridItem(size=QtGui.QVector3D(200,200,0))
    grid.setSize(10,10,10)
    viewWidget.items = [grid]
    addLayer(-20., 0.)
    viewWidget.update()


def showLattice(a, b, c, orders=7, basis=None, shape='Sphere', z0=0, xrot=0, yrot=0, zrot=0, **kwargs):
    vecs = latvec.latticevectors(a, b, c, kwargs['radius'], maxreps=100, repetitions = kwargs['repetitions'], scaling=kwargs['scaling'])

    linez=0

    if basis is None:
        basis = [(0, 0, 0)]

    for basisvec in basis:
        for vec in vecs:
            if shape=='Sphere':
                addSphere(np.sum([map(np.add,vec,[0,0,z0]), basisvec], axis=0),[kwargs['radius']]*3,xrot,yrot,zrot)
                linez=kwargs['radius']
            elif shape=='Box':
                addBox(np.sum([map(np.add,vec,[0,0,z0]), basisvec], axis=0),[kwargs['length'],kwargs['width'],kwargs['height']],xrot,yrot,zrot)
                linez=kwargs['height']
            elif shape == 'Cylinder':
                addCylinder(np.sum([map(np.add, vec, [0, 0, z0]), basisvec], axis=0),
                       [kwargs['radius'], kwargs['radius'], kwargs['height']],xrot,yrot,zrot)
                linez = kwargs['height']


    lines = latvec.latticelines(a, b, c, linez+z0, maxreps=100, repetitions=kwargs['repetitions'], scaling=kwargs['scaling'])

    viewWidget.addItem(latticeFrame(lines))

def addBox(center, scale, xrot, yrot, zrot):
    verts = np.array([[-.5, -.5, .5],
                      [-.5, .5, .5],
                      [.5, -.5, .5],
                      [.5, .5, .5],
                      [-.5, -.5, -.5],
                      [-.5, .5, -.5],
                      [.5, -.5, -.5],
                      [.5, .5, -.5]])
    faces = np.array(
        [[0, 2, 1], [1, 2, 3], [0, 1, 4], [1, 5, 4], [1, 3, 5], [3, 7, 5], [3, 2, 7], [2, 6, 7], [2, 0, 6], [0, 4, 6],
         [4, 5, 6], [5, 7, 6]])

    box = gl.GLMeshItem(vertexes=verts,faces = faces, color=(1,0,1,.3), shader = 'shaded', smooth=True, glOptions='opaque')
    box.rotate(xrot, 1, 0, 0)
    box.rotate(yrot, 0, 1, 0)
    box.rotate(zrot, 0, 0, 1)
    box.translate(*center)
    box.scale(*scale)
    viewWidget.addItem(box)


def cylinder(rows, cols, radius=[1.0, 1.0], length=1.0, offset=False):
    """
    Return a MeshData instance with vertexes and faces computed
    for a cylindrical surface.
    The cylinder may be tapered with different radii at each end (truncated cone)
    """
    verts = np.empty((rows + 1, cols, 3), dtype=float)
    if isinstance(radius, int):
        radius = [radius, radius]  # convert to list
    ## compute vertexes
    th = np.linspace(2 * np.pi, 0, cols).reshape(1, cols)
    r = np.linspace(radius[0], radius[1], num=rows + 1, endpoint=True).reshape(rows + 1, 1)  # radius as a function of z
    verts[..., 2] = np.linspace(0, length, num=rows + 1, endpoint=True).reshape(rows + 1, 1)  # z
    if offset:
        th = th + ((np.pi / cols) * np.arange(rows + 1).reshape(rows + 1, 1))  ## rotate each row by 1/2 column
    verts[..., 0] = r * np.cos(th)  # x = r cos(th)
    verts[..., 1] = r * np.sin(th)  # y = r sin(th)
    verts = verts.reshape((rows + 1) * cols, 3)  # just reshape: no redundant vertices...
    ## compute faces
    faces = np.empty((rows * cols * 2, 3), dtype=np.uint)
    rowtemplate1 = ((np.arange(cols).reshape(cols, 1) + np.array([[0, 1, 0]])) % cols) + np.array([[0, 0, cols]])
    rowtemplate2 = ((np.arange(cols).reshape(cols, 1) + np.array([[0, 1, 1]])) % cols) + np.array([[cols, 0, cols]])
    for row in range(rows):
        start = row * cols * 2
        faces[start:start + cols] = rowtemplate1 + row * cols
        faces[start + cols:start + (cols * 2)] = rowtemplate2 + row * cols

    verts=np.vstack([verts,[0,0,0],[0,0,1]])
    top = ((np.arange(cols).reshape(cols, 1) + np.array([[0, 1, 0]])) % cols) * np.array([1,1,0]) + np.array([[0, 0, len(verts)-2]])
    down = ((np.arange(cols).reshape(cols, 1) + np.array([[0, 1, 1]])) % cols) * np.array([1,1,0]) + np.array([[cols, cols, len(verts)-1]])

    faces=np.vstack([faces,top,down])


    return gl.MeshData(vertexes=verts, faces=faces.astype(np.int))

def addCylinder(center, scale, xrot, yrot, zrot):
    md = cylinder(rows=1, cols=20)
    alpha = .3
    cyl = gl.GLMeshItem(meshdata=md, smooth=True, color=(1, 0, 1, alpha), shader='shaded')
    cyl.translate(0, 0, -scale[2] / 2.)
    cyl.rotate(xrot, 1, 0, 0)
    cyl.rotate(yrot, 0, 1, 0)
    cyl.rotate(zrot, 0, 0, 1)
    cyl.translate(*center)
    cyl.scale(*scale)

    viewWidget.addItem(cyl)

def addSphere(center, scale, xrot, yrot, zrot):
    md = gl.MeshData.sphere(rows=5, cols=10)
    alpha = .3
    sphere = gl.GLMeshItem(meshdata=md, smooth=True, color=(1, 0, 1, alpha), shader='shaded')
    # No rotation for spheres!
    sphere.translate(*center)
    sphere.scale(*scale)
    viewWidget.addItem(sphere)


def addLayer(z0, z1, glOptions='opaque'):
    verts = np.array([[-boxsize, -boxsize, z0],
                      [-boxsize, boxsize, z0],
                      [boxsize, -boxsize, z0],
                      [boxsize, boxsize, z0],
                      [-boxsize, -boxsize, z1],
                      [-boxsize, boxsize, z1],
                      [boxsize, -boxsize, z1],
                      [boxsize, boxsize, z1]])

    faces = np.array(
        [[0, 2, 1], [1, 2, 3], [0, 1, 4], [1, 5, 4], [1, 3, 5], [3, 7, 5], [3, 2, 7], [2, 6, 7], [2, 0, 6], [0, 4, 6],
         [4, 5, 6], [5, 7, 6]])

    layer = gl.GLMeshItem(vertexes=verts, faces=faces, color=(.3, .3, .3, .8), smooth=True, shader='shaded',
                          glOptions=glOptions)
    viewWidget.addItem(layer)

def redraw(*args,**kwargs):
    clear()
    layerz=0
    particlez=0
    for feature in featuremanager.features:
        if type(feature) is customwidgets.layer:
            particlez=layerz
            addLayer(layerz,layerz+feature.Thickness.value()*1.e9,glOptions='additive')
            layerz+=feature.Thickness.value()*1.e9
        elif type(feature) is customwidgets.particle:
            basis = [vecparam.value() for vecparam in feature.structure.Basis.children()]
            showLattice(map(float, feature.structure.LatticeA.value()), map(float, feature.structure.LatticeB.value()),
                            map(float, feature.structure.LatticeC.value()), basis=basis, z0=particlez,
                        shape=feature.Type.value(), radius=feature.Radius.Value.value(), height=feature.Height.Value.value(),
                        width=feature.Width.Value.value(), length=feature.Length.Value.value(), baseangle=feature.BaseAngle.Value.value(),
                        xrot=feature.XRotation.value(),yrot=feature.YRotation.value(),zrot=feature.ZRotation.value(),
                        repetitions=feature.structure.Repetition.value(),scaling=feature.structure.Scaling.value())


import base
from PySide import QtGui, QtCore
from xicam import xglobals
from pipeline import msg
import numpy as np

colors = {msg.DEBUG: QtCore.Qt.gray, msg.ERROR: QtCore.Qt.darkRed, msg.CRITICAL: QtCore.Qt.red,
          msg.INFO: QtCore.Qt.white, msg.WARNING: QtCore.Qt.yellow}


class LogPlugin(base.plugin):
    name = 'Log'
    sigLog = QtCore.Signal(int,str,str,np.ndarray)

    def __init__(self, *args, **kwargs):
        self.centerwidget = QtGui.QListWidget()
        self.rightwidget = None
        self.featureform = None
        self.bottomwidget = None
        self.leftwidget = None
        self.toolbar = None

        msg.guilogcallable = self.log
        msg.flushbacklog()

        super(LogPlugin, self).__init__(*args, **kwargs)

    def log(self, level, timestamp, s, image=None, icon=None):  # We can have icons!
        item = QtGui.QListWidgetItem(s)
        item.setForeground(QtGui.QBrush(colors[level]))
        item.setToolTip(timestamp)
        self.centerwidget.addItem(item)
        if image is not None:
            image=np.uint8((image - image.min())/image.ptp()*255.0)
            pixmap=QtGui.QPixmap.fromImage(QtGui.QImage(image,image.shape[0],image.shape[1],QtGui.QImage.Format_Indexed8))
            i = QtGui.QListWidgetItem()
            w = QtGui.QLabel()
            w.setPixmap(pixmap)
            size = QtCore.QSize(*image.shape)
            w.setFixedSize(size)
            i.setSizeHint(w.sizeHint())
            self.centerwidget.addItem(i)
            self.centerwidget.setItemWidget(i,w)

    # Defer methods to other plugins

    def openSelected(self, *args, **kwargs):
        xglobals.plugins['Timeline'].instance.activate()
        xglobals.plugins['Timeline'].instance.openSelected(*args, **kwargs)

    def openfiles(self, *args, **kwargs):
        xglobals.plugins['Viewer'].instance.activate()
        xglobals.plugins['Viewer'].instance.openfiles(*args, **kwargs)

    def calibrate(self):
        xglobals.plugins['Viewer'].instance.activate()
        xglobals.plugins['Viewer'].instance.calibrate()

    def currentImage(self):
        xglobals.plugins['Viewer'].instance.activate()
        xglobals.plugins['Viewer'].instance.currentImage()



import glob
import os

import numpy as np
import scipy
from scipy import signal
import fabio
import matplotlib.pyplot as plt

import center_approx
import integration

import peakfindingrem
import peakfinding
import scipy.optimize as optimize
import scipy.stats
from scipy import interpolate

from lmfit import minimize, Parameters
import cv2

from scipy.ndimage import filters

from scipy.fftpack import rfft, irfft

from skimage.restoration import denoise_bilateral


demo = True


# def find_arcs(img, cen):
# img = img * (img > 1)
#     img = np.log(img + 1) / np.log(np.max(img)) * 255
#     img = np.array(img, dtype=np.uint8)
#     img = img * Pilatus2M_Mask()
#
#     plt.imshow(img)
#     plt.show()
#     # plt.imshow(img)
#     # plt.show()
#     # search theta
#     Nradial = 25
#
#     arclist = []
#
#     for Theta in np.linspace(-180.0, 0.0, Nradial, endpoint=False):
#         mask = arcmask(img, cen, [0, img.shape[0]], [Theta, Theta + 180.0 / Nradial])
#
#         if demo:
#             plt.imshow(img * mask)
#             plt.show()
#
#         thetaprofile = radial_integrate(mask * img, cen)
#         # print thetaprofile
#
#         peakRs = findpeaks(thetaprofile)
#
#         print peakRs
#
#         if demo:
#             fig = plt.figure()
#
#             h = fig.add_subplot(211)
#             plt.plot(thetaprofile)
#             plt.plot(peakRs, thetaprofile[peakRs], 'r*')
#             limits = h.axis()
#             h = fig.add_subplot(212)
#             plt.imshow(signal.cwt(thetaprofile, signal.ricker, np.arange(5, 100)))
#             cwtlimits = h.axis()
#             h.axis([limits[0], limits[1], cwtlimits[2], cwtlimits[3], ])
#             fig.tight_layout()
#             plt.show()
#
#         for peakR in peakRs:
#             Rwidth = 20
#             mask = arcmask(img, cen, (-0.5 * Rwidth + peakR, 0.5 * Rwidth + peakR), (-180.0, 0.0))
#             if demo:
#                 plt.imshow(img * mask)
#                 plt.show()
#
#             Rprofile = alt_integrate(img * mask, cen)
#
#             Nangle = 50.0
#             peakThetas = np.array(findpeaks(Rprofile))
#
#             if demo:
#                 plt.plot(np.linspace(0, np.pi * 2, Rprofile.__len__()), Rprofile)
#                 plt.plot(peakThetas / Nangle, Rprofile[peakThetas], 'r*')
#                 plt.show()
#
#             peakThetas = peakThetas / Nangle
#             print 'Here', peakThetas
#
#             #add peak to arclist
#             for peakTheta in peakThetas:
#                 arclist.append([peakR * np.cos(peakTheta), peakR * np.sin(peakTheta)])
#
#     arclist = np.array(arclist)
#     plt.imshow(img)
#     plt.plot(cen[0] - arclist[:, 0], cen[1] - arclist[:, 1], 'r*')
#     plt.show()


def findpeaks(Y):
    # Find peaks using continuous wavelet transform; parameter is the range of peak widths (may need tuning)
    # plt.plot(Y)
    # plt.show()
    Y = np.nan_to_num(Y)
    peakindices = scipy.signal.find_peaks_cwt(Y, np.arange(20, 100), noise_perc=5, min_length=10)

    # peakindices=peakutils.indexes(Y, thres=0.6,min_dist=20)

    return peakindices


# def radial_integrate(img, cen):
# # Radial integration
#     y, x = np.indices(img.shape)
#     r = np.sqrt((x - cen[0]) ** 2 + (y - cen[1]) ** 2)
#     r = np.rint(r).astype(np.int)
#
#     tbin = np.bincount(r.ravel(), img.ravel())
#
#     nr = np.bincount(r.ravel(), (img > 0).ravel())
#     radialprofile = tbin / nr
#
#     return radialprofile


# def alt_integrate(img, cen):
# # Radial integration
#     N = 50
#     y, x = np.indices((img.shape))
#     r = (np.arctan2(y - cen[1], x - cen[0]) + np.pi) * N
#     r = np.rint(r).astype(np.int)
#     tbin = np.bincount(r.ravel(), img.ravel())
#     nr = np.bincount(r.ravel(), (img > 0).ravel())
#     radialprofile = tbin / nr
#
#     return radialprofile


# def Pilatus2M_Mask():
# row = 1679
#     col = 1475
#     mask = np.zeros((row, col))
#
#     row_start = 196
#     row_gap_size = 16
#     row_num_gaps = 7
#
#     col_start = 488
#     col_gap_size = 6
#     col_num_gaps = 2
#
#     start = row_start
#     for i in range(1, row_num_gaps + 1):
#         mask[start:start + row_gap_size, :] = 1
#         start = start + row_gap_size + row_start
#     start = col_start
#     for j in range(1, col_num_gaps + 1):
#         mask[:, start:start + col_gap_size] = 1
#         start = start + col_gap_size + col_start
#
#     return 1 - mask


# def arcmask(img, cen, Rrange, Thetarange):
# mask = np.zeros_like(img)
# #print cen, Rrange,Thetarange
# if min(Rrange)==0:
# cv2.ellipse(mask, (int(cen[0]),int(cen[1])), (int(max(Rrange)),int(max(Rrange))), 0, min(Thetarange), int(max(Thetarange)), 255, -1)
#     else:
#         cv2.ellipse(mask, (int(cen[0]),int(cen[1])), (int(min(Rrange)),int(min(Rrange))), 0, min(Thetarange), int(max(Thetarange)), 255, int(max(Rrange)-min(Rrange)))
#     #cv2.ellipse(mask,(256,256),(100,50),0,0,180,255,-1)
#
#     plt.imshow(mask)
#     plt.show()
#     return mask/255

def arcmask(img, cen, Rrange, Thetarange):
    y, x = np.indices((img.shape))
    r = np.sqrt((x - cen[0]) ** 2 + (y - cen[1]) ** 2)
    theta = np.arctan2(y - cen[1], x - cen[0]) / (2 * np.pi) * 360.0
    mask = ((min(Rrange) < r) & (r < max(Rrange)) & (min(Thetarange) < theta) & (theta < max(Thetarange)))
    #plt.imshow(mask)
    #plt.show()

    return mask


def scanforarcs(radialprofile, cen):
    # h = 35
    # radialprofile=signal.convolve(radialprofile,signal.gaussian(h, std=8))
    # test = np.max(radialprofile) / h
    #print 't', test
    peakmax, peakmin = peakfindingrem.peakdet(range(len(radialprofile)), radialprofile, 10)
    peakind = peakmax[:, 0]


    # for i in range(np.size(peakind)):
    # plt.axvline(peakind[i],color='b')
    # plt.plot(radialprofile)
    # plt.show()

    # accurancy = 50
    # x = np.zeros((np.size(peakind), accurancy))
    #y = np.zeros((np.size(peakind), accurancy))
    #xinf = np.zeros((np.size(peakind), accurancy))
    #yinf = np.zeros((np.size(peakind), accurancy))
    #xsup = np.zeros((np.size(peakind), accurancy))
    #ysup = np.zeros((np.size(peakind), accurancy))

    # for i in range(0, np.size(peakind)):
    #Delta = peakind[i] / 10
    #theta = np.linspace(0, 2 * np.pi, accurancy)
    #x[i] = cen[0] + (peakind[i]) * np.cos(theta)
    #y[i] = cen[1] + (peakind[i]) * np.sin(theta)
    #xinf[i] = cen[0] + (peakind[i] - Delta) * np.cos(theta)
    #xsup[i] = cen[0] + (peakind[i] + Delta) * np.cos(theta)
    #yinf[i] = cen[1] + (peakind[i] - Delta) * np.sin(theta)
    #ysup[i] = cen[1] + (peakind[i] + Delta) * np.sin(theta)

    return peakind


def mirroredgaussian(theta, a, b, c, d):
    val = (gaussian(theta, a, b, c, d) + gaussian(2 * np.pi - theta, a, b, c, d)) / 2.
    return val

def gaussian(x, a, b, c, d):
    val = abs(a) * np.exp(-(x - b) ** 2. / c ** 2.) + abs(d)
    return val


def vonmises(x, A, mu, kappa):
    return A * scipy.stats.vonmises.pdf(2 * (x - mu), kappa)


def mirroredvonmises(x, A, mu, kappa, floor):
    return A * (scipy.stats.vonmises.pdf(2 * (mu - x), kappa) + scipy.stats.vonmises.pdf(2 * (mu - x),
                                                                                         kappa)) / 2 + floor  # 2*(mu-(np.pi-x))


tworoot2ln2 = 2. * np.sqrt(2. * np.log(2.))


def residual(params, x, data):
    A = params['A'].value
    mu = params['mu'].value
    kappa = params['kappa'].value
    floor = params['floor'].value

    model = mirroredvonmises(x, A, mu, kappa, floor)

    return data - model


def gaussianresidual(params, x, data, sig=1):
    A = params['A'].value
    mu = params['mu'].value
    sigma = params['sigma'].value
    floor = params['floor'].value

    model = gaussian(x, A, mu, sigma, floor)

    resids = data - model

    # print resids

    weighted = np.sqrt(resids ** 2 / sig ** 2)
    return weighted


def findgisaxsarcs(img, cen, experiment):
    radialprofile = integration.pixel_2Dintegrate(img, (cen[1], cen[0]), experiment.mask)
    # arcs = scanforarcs(radialprofile, cen)
    arcs = peakfinding.findpeaks(None, radialprofile, (100, 50), gaussianwidthsigma=3, minimumsigma=100)
    # print arcs
    plt.plot(radialprofile)
    plt.plot(arcs[0], arcs[1], 'ok')
    #plt.show()
    arcs = arcs[0]

    output = []
    _, unique = np.unique(arcs, return_index=True)

    for qmu in arcs[unique]:
        chiprofile = np.nan_to_num(integration.chi_2Dintegrate(img, (cen[1], cen[0]), qmu, mask=experiment.mask))

        plt.plot(np.arange(0, np.pi, 1 / 30.), chiprofile, 'r')

        # filter out missing chi
        missingpointfloor = np.percentile(chiprofile, 15)
        badpoints = np.where(chiprofile < missingpointfloor)[0]
        goodpoints = np.where(chiprofile >= missingpointfloor)[0]

        chiprofile[badpoints] = np.interp(badpoints, goodpoints, chiprofile[goodpoints])

        plt.plot(np.arange(0, np.pi, 1 / 30.), chiprofile, 'k')

        # f=rfft(chiprofile)
        # plt.plot(f)
        # f[-20:]=0
        # chiprofile=irfft(chiprofile)




        try:
            params = Parameters()
            params.add('A', value=np.max(chiprofile), min=0)
            params.add('mu', value=np.pi / 2, min=0, max=np.pi)
            params.add('kappa', value=0.1, min=0)
            params.add('floor', value=0.1, min=0)
            x = np.arange(0, np.pi, 1 / 30.)

            out = minimize(residual, params, args=(x, chiprofile))
            print params
            # print params['A'].stderr

            # popt, pcov = optimize.curve_fit(vonmises, np.arange(0, np.pi, 1 / 30.), np.nan_to_num(chiprofile),
            #

            # print(popt)
        except RuntimeError:
            print('Fit failed at ' + qmu)
            continue

        if params['kappa'].stderr > 100 or params['A'].stderr > 100:
            isring = True
        else:
            isring = False

        popt = [params['A'].value, params['mu'].value, params['kappa'].value, params['floor'].value]
        A, chimu, kappa, baseline = popt
        FWHM = np.arccos(np.log(.5 * np.exp(kappa) + .5 * np.exp(-kappa)) / kappa)

        output.append([qmu, A, chimu, FWHM, baseline, isring])
        # plt.plot(np.arange(0, np.pi, 1 / 30.), chiprofile)
        # plt.plot(np.arange(0, np.pi, 1 / 30.), mirroredvonmises(np.arange(0, np.pi, 1 / 30.), *popt))
        # plt.show()

    return output


def inpaint(img,mask):
    filled = None
    if False:
        img = img / (2 ^ 16 - 1) * 255
        plt.imshow(img.astype(np.uint8))
        plt.show()

        plt.imshow(mask)  #TODO: check that mask corners are correct
        plt.show()

        kernel = np.ones((3, 3),np.uint8)
        mask = cv2.dilate(mask.astype(np.uint8), kernel, iterations=1)

        filled = cv2.inpaint(img.astype(np.uint8), mask.astype(np.uint8), 3, cv2.INPAINT_TELEA)

        plt.imshow(img)
        plt.show()

        return

    elif True:

        valid = ~mask.astype(np.bool)
        coords = np.array(np.nonzero(valid)).T
        values = img[valid]

        it = interpolate.LinearNDInterpolator(coords, values)

        filled = it(list(np.ndindex(img.shape))).reshape(img.shape)

        plt.imshow(np.rot90(filled))
        plt.show()

    return filled


def findmaxs(orig):
    img = orig.copy()
    img = filters.gaussian_filter(img, 3)
    img = filters.minimum_filter(img, 4)

    img = filters.median_filter(img, 4)

    img -= np.min(img)
    img = denoise_bilateral(img, sigma_range=0.5, sigma_spatial=15)

    # plt.imshow(np.rot90(img))
    #plt.show()

    #img = filters.percentile_filter(img,50,50)

    maxima = ((img == filters.maximum_filter(img, (10, 10))) & (
    filters.maximum_filter(img, (50, 50)) > 1.5 * filters.minimum_filter(img, (50, 50))) & (img > 2))
    maximachis, maximaqs = np.where(maxima == 1)
    plt.imshow(np.rot90(orig), interpolation='nearest')
    plt.plot(maximachis, 1000 - maximaqs, 'o', markersize=10, markeredgecolor='red', markerfacecolor="None", mew="4")
    plt.ylim([1000, 0])
    plt.xlim([0, 1000])
    plt.show()

    return maximachis, maximaqs


def fitarc(chiprofile):
    try:
        params = Parameters()
        params.add('A', value=np.max(chiprofile), min=0)
        params.add('mu', value=np.pi / 2, min=0, max=np.pi)
        params.add('kappa', value=0.1, min=0)
        params.add('floor', value=0.1, min=0)
        x = np.arange(0, np.pi, 1 / 30.)

        out = minimize(residual, params, args=(x, chiprofile))
        print params
        # print params['A'].stderr

        # popt, pcov = optimize.curve_fit(vonmises, np.arange(0, np.pi, 1 / 30.), np.nan_to_num(chiprofile),
        # p0=[np.max(np.nan_to_num(chiprofile)), np.pi / 2, .1, 0])
        # print(popt)
    except RuntimeError:
        print('Fit failed.')

    if params['kappa'].stderr > 100 or params['A'].stderr > 100:
        isring = True
    else:
        isring = False

    popt = [params['A'].value, params['mu'].value, params['kappa'].value, params['floor'].value]
    A, chimu, kappa, baseline = popt
    FWHM = np.arccos(np.log(.5 * np.exp(kappa) + .5 * np.exp(-kappa)) / kappa)

    return A, chimu, FWHM, baseline, isring


def fitarcgaussian(chiprofile, chi):
    try:
        params = Parameters()
        x = np.arange(np.size(chiprofile))
        roi = np.ones_like(chiprofile)
        roi[chi - 30:chi + 30] = .0001
        # roi/=1000
        # plt.plot(chiprofile,'')
        #plt.plot(roi * np.max(chiprofile * roi), 'g')
        #plt.plot(roi*chiprofile,'k')
        params.add('A', value=np.max(chiprofile * (1 - roi)), min=0)
        params.add('mu', value=chi, min=0, max=len(chiprofile))
        params.add('sigma', value=20, min=0)
        params.add('floor', value=0.1, min=0)

        out = minimize(gaussianresidual, params, args=(x, chiprofile, roi), method='nelder')
        #print params
        # print params['A'].stderr

        # popt, pcov = optimize.curve_fit(vonmises, np.arange(0, np.pi, 1 / 30.), np.nan_to_num(chiprofile),
        # p0=[np.max(np.nan_to_num(chiprofile)), np.pi / 2, .1, 0])
        # print(popt)
    except RuntimeError:
        print('Fit failed.')

    if params['sigma'].stderr > 100 or params['A'].stderr > 100:
        isring = False  #True
    else:
        isring = False

    popt = [params['A'].value, params['mu'].value, params['sigma'].value, params['floor'].value]
    #plt.plot(x, gaussian(x, *popt), 'r')
    # plt.show()
    # A, chimu, sigma, baseline = popt
    # FWHM = sigma * tworoot2ln2

    return popt


def findgisaxsarcs2(img, experiment):
    img = img.T.copy()
    cake, _, _ = integration.cake(img, experiment, mask=experiment.mask)  # TODO: refactor these parameters and check .T
    maskcake, _, _ = integration.cake(experiment.mask.T, experiment)

    from fabio import edfimage

    fabimg = edfimage.edfimage(cake)
    filename = 'cake.edf'
    fabimg.write(filename)

    fabimg = edfimage.edfimage(maskcake)
    filename = 'cake_MASK.edf'
    fabimg.write(filename)

    img = inpaint(cake, maskcake)

    fabimg = edfimage.edfimage(img)
    filename = 'cake_LINEAR_INFILL.edf'
    fabimg.write(filename)

    maxchis, maxqs = findmaxs(img)

    out =[]

    for chi, q in zip(maxchis, maxqs):
        # roi=np.ones_like(img)
        #roi[chi - 10:chi + 10, q - 5:q + 5]=10
        #roi=np.sum(roi,axis=1)
        slice = img[:, q - 5:q + 5]
        if np.max(slice) / np.min(slice) < 2:
            pass  # continue
        chiprofile = np.sum(slice, axis=1)
        x = np.arange(np.size(chiprofile))

        #plt.plot(chiprofile)

        params = fitarcgaussian(chiprofile, chi)
        if params['mu'] > chi + 5 or params['mu'] < chi - 5:
            continue
        params.add('q', value=q)
        out.append(params)



        #plt.show()

    # plt.imshow(np.log(img))
    #plt.show()

    return out


if __name__ == "__main__":
    import xicam.config

    experiment = xicam.config.experiment()
    experiment.setvalue('Detector', 'pilatus2m')
    experiment.setvalue('Pixel Size X',172e-6)
    experiment.setvalue('Pixel Size Y', 172e-6)
    experiment.mask = experiment.getDetector().calc_mask()

    for imgpath in glob.glob(os.path.join("../GISAXS samples/", '*.edf')):
        print "Opening", imgpath

        # read image
        img = fabio.open(imgpath).data
        # find center
        # cen = center_approx.center_approx(img)



        cen = center_approx.gisaxs_center_approx(img)
        experiment.setcenter(cen)

        arcs = findgisaxsarcs2(img, experiment)
        # print cen
        # print arcs

        ax = plt.gca()

        plt.axvline(cen[0], color='r')
        plt.axhline(cen[1], color='r')
        plt.imshow(np.log(img))



        from matplotlib.patches import Arc

        qratio =1.78

        for arc in arcs:
            print arc
            if not np.isnan(arc['sigma'].value):

                if False:
                    arcartist = [Arc(xy=cen, width=arc['q'] * 2, height=arc['q'] * 2, angle=-90, theta1=0,
                                     theta2=360)]  # Arc
                    ax.add_artist(arcartist[0])
                    arcartist[0].set_lw(3)
                else:
                    angle = -arc['mu'].value / 1000 * 360
                    theta1 = -abs(arc['sigma'].value * tworoot2ln2) / 1000 * 360 / 2
                    theta2 = abs(arc['sigma'].value * tworoot2ln2) / 1000 * 360 / 2
                    arcartist = [
                        Arc(xy=cen, width=arc['q'].value * 2 * qratio, height=arc['q'].value * 2 * qratio, angle=angle,
                            theta1=theta1,
                            theta2=theta2)]  # Arc
                    for artist in arcartist:
                        ax.add_artist(artist)
                        artist.set_lw(3)


        # for i in range(1, np.size(x, 0)):
        # plt.plot(y[i], x[i], color='g')
        #     plt.plot(yinf[i], xinf[i], color='r')
        #     plt.plot(ysup[i], xsup[i], color='r')
        plt.show()

        # popt, pcov = optimize.curve_fit(gaussian, np.arange(np.size(a)), np.nan_to_num(a))

        # print("Scale =  %.3f +/- %.3f" % (popt[0], np.sqrt(pcov[0, 0])))
        #print("Offset = %.3f +/- %.3f" % (popt[1], np.sqrt(pcov[1, 1])))
        #print("Sigma =  %.3f +/- %.3f" % (popt[2], np.sqrt(pcov[2, 2])))

        # print(vimodel.A)
        # print vimodel.mu
        # print vimodel.FWHM




        # find arcs
        # arcs = find_arcs(img, cen)

        #draw arcs
        #drawarcs(img,arcs)





__author__ = "Ronald J Pandolfi"
__copyright__ = "Copyright 2016, CAMERA, LBL, ALS"
__credits__ = ["Ronald J Pandolfi", "Dinesh Kumar", "Singanallur Venkatakrishnan", "Luis Luque", "Alexander Hexemer"]
__license__ = ""
__version__ = "1.2.1"
__maintainer__ = "Ronald J Pandolfi"
__email__ = "ronpandolfi@lbl.gov"
__status__ = "Beta"


import os
import imageio
import numpy as np
import pyqtgraph as pg
from PySide import QtCore, QtGui
from vispy import scene
from vispy.color import Colormap
from pipeline import loader
from xicam.widgets.customwidgets import histDialogButton


class VolumeViewer(QtGui.QWidget):

    sigImageChanged=QtCore.Signal()

    def __init__(self,path=None,data=None,*args,**kwargs):
        super(VolumeViewer, self).__init__()

        self.levels = [0, 1]

        ly = QtGui.QHBoxLayout()
        ly.setContentsMargins(0,0,0,0)
        ly.setSpacing(0)

        self.volumeRenderWidget= VolumeRenderWidget()
        ly.addWidget(self.volumeRenderWidget.native)

        self.HistogramLUTWidget = None
        self.HistogramLUTWidget = pg.HistogramLUTWidget(image=self, parent=self)
        self.HistogramLUTWidget.setMaximumWidth(self.HistogramLUTWidget.minimumWidth()+15)# Keep static width
        self.HistogramLUTWidget.setMinimumWidth(self.HistogramLUTWidget.minimumWidth()+15)

        v = QtGui.QVBoxLayout()
        v.addWidget(self.HistogramLUTWidget)

        # ly.addWidget(self.HistogramLUTWidget)

        self.setButton = histDialogButton('Set', parent=self)
        self.setButton.setMaximumWidth(self.HistogramLUTWidget.minimumWidth() + 15)
        self.setButton.setMinimumWidth(self.HistogramLUTWidget.minimumWidth() + 15)
        self.setButton.connectToHistWidget(self.HistogramLUTWidget)
        v.addWidget(self.setButton)
        ly.addLayout(v)

        self.xregion = SliceWidget(parent=self)
        self.yregion = SliceWidget(parent=self)
        self.zregion = SliceWidget(parent=self)
        self.xregion.item.region.setRegion([0, 1000])
        self.yregion.item.region.setRegion([0, 1000])
        self.zregion.item.region.setRegion([0, 1000])
        self.xregion.sigSliceChanged.connect(self.setVolume) #change to setVolume
        self.yregion.sigSliceChanged.connect(self.setVolume)
        self.zregion.sigSliceChanged.connect(self.setVolume)
        ly.addWidget(self.xregion)
        ly.addWidget(self.yregion)
        ly.addWidget(self.zregion)

        self.setLayout(ly)

        # self.setVolume(vol=data,path=path)

        # self.volumeRenderWidget.export('video.mp4',fps=25,duration=10.)
        # self.writevideo()


    @property
    def vol(self):
        return self.volumeRenderWidget.vol

    def getSlice(self):
        xslice=self.xregion.getSlice()
        yslice=self.yregion.getSlice()
        zslice=self.zregion.getSlice()
        return xslice,yslice,zslice

    def setVolume(self, vol=None, path=None, slicevol=True):
        if slicevol:
            sliceobj = self.getSlice()
            print 'Got slice', sliceobj
        else:
            sliceobj = 3*(slice(0, None),)

        if vol is not None: vol=np.nan_to_num(vol)

        self.volumeRenderWidget.setVolume(vol, path, sliceobj)
        self.volumeRenderWidget.update()
        if vol is not None or path is not None:
            self.sigImageChanged.emit()
            for i, region in enumerate([self.xregion, self.yregion, self.zregion]):
                try:
                    region.item.region.setRegion([0, vol.shape[i]])
                except RuntimeError as e:
                    print e.message

    def moveGradientTick(self, idx, pos):
        tick = self.HistogramLUTWidget.item.gradient.listTicks()[idx][0]
        tick.setPos(pos, 0)
        tick.view().tickMoved(tick, QtCore.QPoint(pos*self.HistogramLUTWidget.item.gradient.length, 0))
        tick.sigMoving.emit(tick)
        tick.sigMoved.emit(tick)
        tick.view().tickMoveFinished(tick)

    def setLevels(self, levels, update=True):
        self.levels = levels
        self.setLookupTable()
        if self.HistogramLUTWidget: self.HistogramLUTWidget.region.setRegion(levels)
        if update:
            self.volumeRenderWidget.update()

    def setLookupTable(self, lut=None, update=True):
        try:
            table = self.HistogramLUTWidget.item.gradient.colorMap().color/256.
            pos = self.HistogramLUTWidget.item.gradient.colorMap().pos
            #table=np.clip(table*(self.levels[1]-self.levels[0])+self.levels[0],0.,1.)
            table[:, 3] = pos
            table = np.vstack([np.array([[0,0,0,0]]),table,np.array([[1,1,1,1]])])
            pos = np.hstack([[0], pos*(self.levels[1] - self.levels[0]) + self.levels[0], [1]])
            self.volumeRenderWidget.volume.cmap = Colormap(table, controls=pos)
        except AttributeError as ex:
            print ex


    def getHistogram(self, bins='auto', step='auto', targetImageSize=100, targetHistogramSize=500, **kwds):
        """Returns x and y arrays containing the histogram values for the current image.
        For an explanation of the return format, see numpy.histogram().

        The *step* argument causes pixels to be skipped when computing the histogram to save time.
        If *step* is 'auto', then a step is chosen such that the analyzed data has
        dimensions roughly *targetImageSize* for each axis.

        The *bins* argument and any extra keyword arguments are passed to
        np.histogram(). If *bins* is 'auto', then a bin number is automatically
        chosen based on the image characteristics:

        * Integer images will have approximately *targetHistogramSize* bins,
          with each bin having an integer width.
        * All other types will have *targetHistogramSize* bins.

        This method is also used when automatically computing levels.
        """
        if self.vol is None:
            return None,None
        if step == 'auto':
            step = (int(np.ceil(float(self.vol.shape[0]) / targetImageSize)),
                    int(np.ceil(float(self.vol.shape[1]) / targetImageSize)))
        if np.isscalar(step):
            step = (step, step)
        stepData = self.vol[::step[0], ::step[1]]

        if bins == 'auto':
            if stepData.dtype.kind in "ui":
                mn = stepData.min()
                mx = stepData.max()
                step = np.ceil((mx-mn) / 500.)
                bins = np.arange(mn, mx+1.01*step, step, dtype=np.int)
                if len(bins) == 0:
                    bins = [mn, mx]
            else:
                bins = 500

        kwds['bins'] = bins
        hist = np.histogram(stepData, **kwds)

        return hist[1][:-1], hist[0]

    # @volumeRenderWidget.connect
    # def on_frame(self,event):
    #     self.volumeRenderWidget.cam1.auto_roll

    def writevideo(self,fps=25):
        writer = imageio.save('foo.mp4', fps=25)
        self.volumeRenderWidget.events.draw.connect(lambda e: writer.append_data(self.render()))
        self.volumeRenderWidget.events.close.connect(lambda e: writer.close())


class VolumeVisual(scene.visuals.Volume):
    def set_data(self, vol, clim=None):
        """ Set the volume data.

        Parameters
        ----------
        vol : ndarray
            The 3D volume.
        clim : tuple | None
            Colormap limits to use. None will use the min and max values.
        """
        # Check volume
        if not isinstance(vol, np.ndarray):
            raise ValueError('Volume visual needs a numpy array.')
        if not ((vol.ndim == 3) or (vol.ndim == 4 and vol.shape[-1] <= 4)):
            raise ValueError('Volume visual needs a 3D image.')

        # Handle clim
        if clim is not None:
            clim = np.array(clim, float)
            if not (clim.ndim == 1 and clim.size == 2):
                raise ValueError('clim must be a 2-element array-like')
            self._clim = tuple(clim)
        self._clim = vol.min(), vol.max()   #NOTE: THIS IS MODIFIED BY RP TO RESET MIN/MAX EACH TIME

        # Apply clim
        vol = np.array(vol, dtype='float32', copy=False)
        vol -= self._clim[0]
        vol *= 1./(self._clim[1] - self._clim[0])

        # Apply to texture
        self._tex.set_data(vol)  # will be efficient if vol is same shape
        self._program['u_shape'] = vol.shape[2], vol.shape[1], vol.shape[0]
        self._vol_shape = vol.shape[:3]

        # Create vertices?
        if self._index_buffer is None:
            self._create_vertex_data()


class SliceWidget(pg.HistogramLUTWidget):
    sigSliceChanged = QtCore.Signal()

    def __init__(self, *args, **kwargs):
        super(SliceWidget, self).__init__(*args, **kwargs)
        self.item.paint = lambda *x: None
        self.item.vb.deleteLater()
        self.item.gradient.gradRect.hide()
        self.item.gradient.allowAdd = False
        self.setMinimumWidth(70)
        self.setMaximumWidth(70)
        self.item.sigLookupTableChanged.connect(self.ticksChanged)
        self.setSizePolicy(QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Expanding)

    def sizeHint(self):
        return QtCore.QSize(70, 200)

    def ticksChanged(self,LUT):
        self.sigSliceChanged.emit()
        #tuple(sorted(LUT.gradient.ticks.values()))

    def getSlice(self):
        bounds = sorted(self.item.gradient.ticks.values())
        bounds = (bounds[0]*self.item.region.getRegion()[1],bounds[1]*self.item.region.getRegion()[1])
        return slice(*bounds)

scene.visuals.Volume = VolumeVisual

class VolumeRenderWidget(scene.SceneCanvas):

    def __init__(self,vol=None, path=None, size=(800,600), show=False):
        super(VolumeRenderWidget, self).__init__(keys='interactive', size=size, show=show)

        # Prepare canvas
        self.measure_fps()

        #self.unfreeze()

        # Set up a viewbox to display the image with interactive pan/zoom
        self.view = self.central_widget.add_view()

        self.vol=None
        self.setVolume(vol,path)
        self.volume=None

        # Create three cameras (Fly, Turntable and Arcball)
        fov = 60.
        self.cam1 = scene.cameras.FlyCamera(parent=self.view.scene, fov=fov, name='Fly')
        self.cam2 = scene.cameras.TurntableCamera(parent=self.view.scene, fov=fov, name='Turntable')
        self.cam3 = scene.cameras.ArcballCamera(parent=self.view.scene, fov=fov, name='Arcball')
        self.view.camera = self.cam2  # Select turntable at first


    def setVolume(self, vol=None, path=None, sliceobj=None):

        if path is not None and vol is None:
            if '*' in path:
                vol = loader.loadimageseries(path)
            elif os.path.splitext(path)[-1]=='.npy':
                vol = loader.loadimage(path)
            else:
                vol = loader.loadtiffstack(path)
        elif vol is None:
            vol = self.vol

        if vol is None:
            return

        self.vol = vol

        if slice is not None:
            def intify(a):
                if a is not None: return int(a)
            sliceobj = [slice(intify(s.start),intify(s.stop),intify(s.step)) for s in sliceobj]
            slicevol = self.vol[sliceobj]
        else:
            slicevol = self.vol

        # Set whether we are emulating a 3D texture
        emulate_texture = False

        # Create the volume visuals
        if self.volume is None:
            self.volume = scene.visuals.Volume(slicevol, parent=self.view.scene, emulate_texture=emulate_texture)
            self.volume.method = 'translucent'
        else:
            self.volume.set_data(slicevol)
            self.volume._create_vertex_data() #TODO: Try using this instead of slicing array?

        # Translate the volume into the center of the view (axes are in strange order for unkown )
        scale = 3*(2.0/self.vol.shape[1],)
        translate = map(lambda x: -scale[0]*x/2, reversed(vol.shape))
        self.volume.transform = scene.STTransform(translate=translate, scale=scale)

    # Implement key presses
    def on_key_press(self, event):
        if event.text == '1':
            cam_toggle = {self.cam1: self.cam2, self.cam2: self.cam3, self.cam3: self.cam1}
            self.view.camera = cam_toggle.get(self.view.camera, self.cam2)
            print(self.view.camera.name + ' camera')
        elif event.text == '2':
            pass
        elif event.text == '3':
            pass
        elif event.text == '4':
            pass
        elif event.text == '0':
            self.cam1.set_range()
            self.cam3.set_range()
        elif event.text != '' and event.text in '[]':
            s = -0.025 if event.text == '[' else 0.025
            self.volume.threshold += s
            th = self.volume.threshold
            print("Isosurface threshold: %0.3f" % th)



# -*- coding: UTF-8 -*-

# TODO: Make experiment save/load
### TODO: Add peak marking
## TODO: Add mask clear



import os

from PySide.QtUiTools import QUiLoader
from PySide import QtGui
from PySide import QtCore

from xicam import config
import watcher
#import daemon
import pipeline
import qdarkstyle
import plugins
from xicam import xglobals
import numpy as np

# import client.dask_io_loop
# import client.dask_local_scheduler
# import client.dask_remote_scheduler
# import client.dask_active_executor
import threads
from pipeline import msg

class ComboBoxAction(QtGui.QWidgetAction):
    def __init__(self, title, parent=None):
        QtGui.QWidgetAction.__init__(self, parent)
        pWidget = QtGui.QWidget()
        pLayout = QtGui.QHBoxLayout()
        pLabel = QtGui.QLabel(title)
        pLayout.addWidget(pLabel)

        pComboBox = QtGui.QComboBox()
        pLayout.addWidget(pComboBox)
        pWidget.setLayout(pLayout)

        self.setDefaultWidget(pWidget)

        # def comboBox (self):
        #    return self.pComboBox


class Login(QtGui.QDialog):
    def __init__(self, machineName="", parent=None):
        super(Login, self).__init__(parent)
        self.textMachine = QtGui.QLineEdit(self)
        self.textMachine.setPlaceholderText("Machine...")
        self.textMachine.setText(machineName)
        self.textName = QtGui.QLineEdit(self)
        self.textName.setPlaceholderText("Username...")
        self.textPass = QtGui.QLineEdit(self)
        self.textPass.setPlaceholderText("Password (Empty for SSH Key)...")
        self.textPass.setEchoMode(QtGui.QLineEdit.Password)
        self.buttonLogin = QtGui.QPushButton('Login', self)
        self.buttonLogin.clicked.connect(self.handleLogin)
        layout = QtGui.QVBoxLayout(self)
        layout.addWidget(self.textMachine)
        layout.addWidget(self.textName)
        layout.addWidget(self.textPass)
        layout.addWidget(self.buttonLogin)
        self.resize(300, 100)

    def handleLogin(self):
        if len(self.textName.text()) > 0 and len(self.textMachine.text()) > 0:
            self.accept()
        else:
            self.close()

class MyMainWindow(QtCore.QObject):
    def __init__(self, app):
        QtCore.QObject.__init__(self, app)
        print 'Gui:\t\t\t', QtGui.QApplication.instance().thread()
        QtGui.QFontDatabase.addApplicationFont("xicam/gui/zerothre.ttf")

        import plugins

        config.activate()

        self._pool = None
        # Load the gui from file
        self.app = app
        guiloader = QUiLoader()
        f = QtCore.QFile("xicam/gui/mainwindow.ui")
        f.open(QtCore.QFile.ReadOnly)
        self.ui = guiloader.load(f)
        f.close()

        # STYLE
        with open('xicam/gui/style.stylesheet', 'r') as f:
            style = f.read()
        app.setStyleSheet(qdarkstyle.load_stylesheet() + style)
        app.setAttribute(QtCore.Qt.AA_DontShowIconsInMenus, False)

        # INITIAL GLOBALS
        self.viewerprevioustab = -1
        self.timelineprevioustab = -1
        self.experiment = config.experiment()
        self.folderwatcher = watcher.newfilewatcher()
        self.plugins = []

        # ACTIONS
        # Wire up action buttons
        self.ui.findChild(QtGui.QAction, 'actionOpen').triggered.connect(self.dialogopen)
        self.ui.findChild(QtGui.QAction, 'actionSettings').triggered.connect(self.settingsopen)
        self.ui.findChild(QtGui.QAction, 'actionQuit').triggered.connect(QtGui.QApplication.instance().quit)

        self.ui.actionExport_Image.triggered.connect(self.exportimage)

        # Grab status bar
        msg.statusbar = self.ui.statusbar
        pb = QtGui.QProgressBar()
        pb.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Ignored)
        msg.progressbar = pb
        pb.setAccessibleName('progressbar')
        msg.statusbar.addPermanentWidget(pb)
        pb.hide()
        msg.showMessage('Ready...')
        xglobals.statusbar = self.ui.statusbar  # TODO: Deprecate this by replacing all statusbar calls with msg module

        # PLUG-INS

        placeholders = [self.ui.viewmode, self.ui.sidemode, self.ui.bottommode, self.ui.toolbarmode, self.ui.leftmode]

        plugins.initplugins(placeholders)

        plugins.plugins['MOTD'].instance.activate()

        plugins.base.fileexplorer.sigOpen.connect(self.openfiles)
        plugins.base.fileexplorer.sigFolderOpen.connect(self.openfolder)
        plugins.base.booltoolbar.actionTimeline.triggered.connect(plugins.base.filetree.handleOpenAction)

        pluginmode = plugins.widgets.pluginModeWidget(plugins.plugins)
        self.ui.modemenu.addWidget(pluginmode)

        self.ui.menubar.addMenu(plugins.buildactivatemenu(pluginmode))

        # TESTING
        ##
        # self.openimages(['../samples/AgB_00016.edf'])
        # self.openimages(['/Users/rp/Data/LaB6_Ant1_dc002p.mar3450'])

        # self.calibrate()
        # self.updatepreprocessing()
        ##
        testmenu = QtGui.QMenu('Testing')
        testmenu.addAction('Single frame').triggered.connect(self.singletest)
        testmenu.addAction('Image stack').triggered.connect(self.stacktest)
        testmenu.addAction('Timeline').triggered.connect(self.timelinetest)
        testmenu.addAction('Tilt').triggered.connect(self.tilttest)

        self.ui.menubar.addMenu(testmenu)

        # DASK WORKFLOW
        # TODO turn this into a class

        # convert the following into a class
        self.sessions = ["localhost", "Andromeda", "Daint", "NERSC/Edison"]
        self.session_machines = ["localhost", "andromeda.dhcp.lbl.gov", "148.187.1.7", "edison.nersc.gov"]
        # self.session_address = ["localhost", socket.gethostbyname("andromeda.dhcp.lbl.gov"), "148.187.26.16", ""]
        self.session_address = ["localhost", "andromeda.dhcp.lbl.gov", "148.187.26.16", ""]
        self.session_exec = ["", "/home/hari/runscript.sh", "/users/course79/runscript.sh",
                             "/usr/common/graphics/visit/camera/runscript.sh"]
        self.executors = [None, None, None, None]

        self.sessionmenu = QtGui.QMenu('Sessions')

        # comboBoxAction = ComboBoxAction("Active Session", self.sessionmenu);

        self.actionGroup = QtGui.QActionGroup(self.sessionmenu)
        for i in self.sessions:
            action = QtGui.QAction(i, self.sessionmenu, checkable=True)
            if i == "localhost":
                action.setChecked(True)
            action.triggered.connect(self.activesessionchanged)
            self.actionGroup.addAction(action)
            self.sessionmenu.addAction(action)

        # self.comboBoxAction.comboBox().activated.connect(self.activesessionchanged)
        # self.sessionmenu.addAction(comboBoxAction)
        self.ui.menubar.addMenu(self.sessionmenu)

        # self.daskLoop = client.dask_io_loop.DaskLoop()
        # try:
        #     # create a local active executor
        #     local_scheduler = client.dask_local_scheduler.LocalScheduler(self.daskLoop)
        #     local_scheduler.execute()
        #     self.executors[0] = local_scheduler
        #     self.sessionmenu.setTitle("Active Session (localhost)")
        #     client.dask_active_executor.active_executor = local_scheduler
        # except:
        #     msg.logMessage("Issues connecting to localhost",msg.ERROR)

        # START PYSIDE MAIN LOOP
        # Show UI and end app when it closes
        self.ui.installEventFilter(self)

    def eventFilter(self, obj, ev):
        if obj is self.ui:  # if the object is from the MainWindow
            if ev.type() == QtCore.QEvent.Close:
                self.closeAllConnections()
                QtGui.QApplication.quit()
                threads.worker.stop()  # ask worker to stop nicely
                #threads.worker.wait()

                return True
            else:
                return False
        return QtCore.QObject.eventFilter(self, obj, ev)

    def closeAllConnections(self):
        msg.logMessage("Closing all connections")

        # stop any existing executors
        # for e in range(len(self.executors)):
        #     if self.executors[e] is not None:
        #         self.executors[e].close()
        # self.daskLoop.loop.stop()
        # self.daskLoop.loop.close()

        # self.daskLoop.loop.instance().add_callback(self.daskLoop.loop.instance().stop)

    def activesessionchanged(self):
        # w = self
        obj = 0
        for (i, ac) in enumerate(self.actionGroup.actions()):
            if self.sender().text() == ac.text():
                obj = i
                break

        # if self.executors[obj] != None:
        #     client.dask_active_executor.active_executor = self.executors[obj]
        #     self.sessionMenu.setText("Active Session ({0})".format(self.session_machines[obj]))
        # else:
        #     # setup connection
        #     login = Login(self.session_machines[obj])
        #     if login.exec_() == QtGui.QDialog.Accepted:
        #         username = str(login.textName.text())
        #         machine = str(login.textMachine.text())
        #         password = str(login.textPass.text())
        #         msg.logMessage((username, machine),msg.DEBUG)  # , password
        #         self.executors[obj] = client.dask_remote_scheduler.RemoteScheduler(machine, username, self.daskLoop,
        #                                                                            password, self.session_address[obj],
        #                                                                            self.session_exec[obj])
        #         self.sessionmenu.setTitle("Active Session ({0})".format(self.session_machines[obj]))
        #
        #         import time
        #         time.sleep(5)
        #         self.executors[obj].execute()
        #         client.dask_active_executor.active_executor = self.executors[obj]

    def singletest(self):
        self.openfiles(['/home/rp/data/3pt8m_gisaxs/26_pt10_30s_hi_2m.edf'])

    def stacktest(self):
        self.openfiles(['/tmp/20140905_191647_YL1031_.h5'])

    def timelinetest(self):
        import glob
        self.openfiles(sorted(glob.glob('/home/rp/data/YL1031/YL1031*.edf')))

    def tilttest(self):
        config.activeExperiment.setvalue('Detector Distance', 2.46269726489 * 79 * .001)
        config.activeExperiment.setvalue('Detector Rotation', 4.69729438873 * 360. / (2. * np.pi) - 180.)
        config.activeExperiment.setvalue('Detector Tilt', 0.503226642865 / (2. * np.pi) * 360.)
        config.activeExperiment.setvalue('Wavelength', 0.97621599151 * 1.e-10)
        config.activeExperiment.setvalue('Center X', 969.878684978)
        config.activeExperiment.setvalue('Center Y', 2048 - 2237.93277884)
        self.openfiles(['/home/rp/Downloads/lab6_041016_rct5_0001.tif'])

    def changetimelineoperation(self, index):
        self.currentTimelineTab().tab.setvariationmode(index)

    @staticmethod
    def load_image(path):
        """
        load an image with fabio
        """
        # Load an image path with fabio
        return pipeline.loader.loadpath(path)[0]

    def dialogopen(self):
        """
        Open a file dialog then open that image
        """
        filename, ok = QtGui.QFileDialog.getOpenFileName(self.ui, 'Open file', os.curdir,
                                                         '*' + ' *'.join(pipeline.loader.acceptableexts))
        if filename and ok:
            self.openfiles([filename])

    def openfiles(self, filenames):
        """
        when a file is opened, check if there is calibration and offer to use the image as calibrant
        """
        if filenames is not u'':
            if config.activeExperiment.iscalibrated or len(filenames) > 1:
                self.openimages(filenames)
            else:
                msgBox = QtGui.QMessageBox()
                msgBox.setText("The current experiment has not yet been calibrated. ")
                msgBox.setInformativeText("Use this image as a calibrant (AgBe)?")
                msgBox.setStandardButtons(QtGui.QMessageBox.Yes | QtGui.QMessageBox.No | QtGui.QMessageBox.Cancel)
                msgBox.setDefaultButton(QtGui.QMessageBox.Yes)

                response = msgBox.exec_()

                if response == QtGui.QMessageBox.Yes:
                    self.openimages(filenames)

                    self.calibrate()
                elif response == QtGui.QMessageBox.No:
                    self.openimages(filenames)
                elif response == QtGui.QMessageBox.Cancel:
                    return None

    def openfolder(self, filenames):
        """
        build a new tab from the folder path, add it to the tab view, and display it
        """

        if filenames is not u'':
            if config.activeExperiment.iscalibrated or len(filenames) > 1:
                import plugins

                self.ui.statusbar.showMessage('Loading images from folder...')
                self.app.processEvents()
                plugins.base.activeplugin.opendirectory(filenames)

                self.ui.statusbar.showMessage('Ready...')

    def exportimage(self):
        plugins.base.activeplugin.exportimage()

    def calibrate(self):

        """
        Calibrate using the currently active tab
        """
        # self.currentImageTab().load()
        plugins.base.activeplugin.calibrate()

    def openimages(self, paths):
        """
        build a new tab, add it to the tab view, and display it
        """

        import plugins

        self.ui.statusbar.showMessage('Loading image...')
        self.app.processEvents()
        # Make an image tab for that file and add it to the tab view
        # newimagetab = viewer.imageTabTracker([path], self.experiment, self)
        #tabwidget = self.ui.findChild(QtGui.QTabWidget, 'tabWidget')
        #tabwidget.setCurrentIndex(tabwidget.addTab(newimagetab, path.split('/')[-1]))
        plugins.base.activeplugin.openfiles(paths)

        self.ui.statusbar.showMessage('Ready...')

    def loadexperiment(self):
        """
        replot the current tab (tab plotting checks if this is active)
        """
        path, _ = QtGui.QFileDialog.getOpenFileName(self.ui, 'Open file', os.curdir, "*.exp")
        self.experiment = config.experiment(path)

    def updateexperiment(self):
        self.experiment.save()
        if hasattr(self.currentImageTab(), 'tab'):
            if self.currentImageTab().tab is not None:
                self.currentImageTab().tab.redrawimage()
                self.currentImageTab().tab.drawcenter()
                self.currentImageTab().tab.replot()
                self.currentImageTab().tab.dimg.invalidatecache()

    def filebrowserpanetoggle(self):
        """
        toggle this pane as visible/hidden
        """
        pane = self.ui.findChild(QtGui.QTreeView, 'treebrowser')
        pane.setHidden(not pane.isHidden())

    def openfilestoggle(self):
        """
        toggle this pane as visible/hidden
        """
        pane = self.ui.findChild(QtGui.QListView, 'openfileslist')
        pane.setHidden(not pane.isHidden())

    def watchfoldtoggle(self):
        """
        toggle this pane as visible/hidden
        """
        pane = self.ui.findChild(QtGui.QFrame, 'watchframe')
        pane.setVisible(not pane.isVisible())

    def experimentfoldtoggle(self):
        """
        toggle this pane as visible/hidden
        """
        pane = self.experimentTree
        pane.setHidden(not pane.isHidden())

    def propertiesfoldtoggle(self):
        """
        toggle this pane as visible/hidden
        """
        pane = self.ui.propertytable
        pane.setHidden(not pane.isHidden())

    def openwatchfolder(self):
        """
        Start a daemon thread watching the selected folder
        """
        dialog = QtGui.QFileDialog(self.ui, 'Choose a folder to watch', os.curdir,
                                   options=QtGui.QFileDialog.ShowDirsOnly)
        d = dialog.getExistingDirectory()
        if d:
            self.ui.findChild(QtGui.QLabel, 'watchfolderpath').setText(d)
            self.folderwatcher.addPath(d)
            if self.ui.findChild(QtGui.QCheckBox, 'autoPreprocess').isChecked():
                self.daemonthread = daemon.daemon(d, self.experiment, procold=True)
                self.daemonthread.start()

    def resetwatchfolder(self):
        """
        Resets the watch folder gui and ends current daemon
        """
        self.folderwatcher.removePaths(self.folderwatcher.directories())
        self.ui.findChild(QtGui.QLabel, 'watchfolderpath').setText('')
        self.daemonthread.stop()

    def newfilesdetected(self, d, paths):
        """
        When new files are detected, auto view/timeline them
        """
        # TODO: receive data from daemon thread instead of additional watcher object.
        if self.ui.findChild(QtGui.QCheckBox, 'autoView').isChecked():
            for path in paths:
                msg.logMessage(path,msg.INFO)
                self.openfile(os.path.join(d, path))
        if self.ui.findChild(QtGui.QCheckBox, 'autoTimeline').isChecked():
            self.showtimeline()
            if self.currentTimelineTab() is None:
                newtimelinetab = timeline.timelinetabtracker([], self.experiment, self)
                timelinetabwidget = self.ui.findChild(QtGui.QTabWidget, 'timelinetabwidget')

                timelinetabwidget.setCurrentIndex(timelinetabwidget.addTab(newtimelinetab, 'Watched timeline'))
                self.currentTimelineTab().load()
            self.currentTimelineTab().tab.appendimage(d, paths)

            # def loadhiprmc(self):
            #     self.loadplugin(hiprmc)
            #
            # def loadplugin(self,module):

    def settingsopen(self):
        config.settings.showEditor()


import platform
from fabio import edfimage

# Use NSURL as a workaround to pyside/Qt4 behaviour for dragging and dropping on OSx
op_sys = platform.system()
# if op_sys == 'Darwin':
#     from Foundation import NSURL

import base
from PySide import QtGui
import os

import widgets
import numpy as np
from pipeline.spacegroups import spacegroupwidget
from pipeline import loader


class XASPlugin(base.plugin):
    name = 'XAS'

    def __init__(self, *args, **kwargs):

        self.centerwidget = QtGui.QTabWidget()
        self.centerwidget.currentChanged.connect(self.currentChanged)
        self.centerwidget.setDocumentMode(True)
        self.centerwidget.setTabsClosable(True)
        self.centerwidget.tabCloseRequested.connect(self.tabCloseRequested)

        super(XASPlugin, self).__init__(*args, **kwargs)

        # DRAG-DROP
        self.centerwidget.setAcceptDrops(True)
        self.centerwidget.dragEnterEvent = self.dragEnterEvent
        self.centerwidget.dropEvent = self.dropEvent


    def dragEnterEvent(self, e):
        print(e)
        e.accept()
        # if e.mimeData().hasFormat('text/plain'):
        # e.accept()
        # else:
        #     e.accept()

    def dropEvent(self, e):
        for url in e.mimeData().urls():
            if op_sys == 'Darwin':
                fname = str(NSURL.URLWithString_(str(url.toString())).filePathURL().path())
            else:
                fname = str(url.toLocalFile())
            if os.path.isfile(fname):
                print(fname)
                self.openfiles([fname])
            e.accept()




    def drawsgoverlay(self, peakoverlay):
        self.getCurrentTab().drawsgoverlay(peakoverlay)

    def addmode(self):
        """
        Launch a tab as an add operation
        """
        operation = lambda m: np.sum(m, 0)
        self.launchmultimode(operation, 'Addition')

    def subtractmode(self):
        """
        Launch a tab as an sub operation
        """
        operation = lambda m: m[0] - np.sum(m[1:], 0)
        self.launchmultimode(operation, 'Subtraction')

    def addwithcoefmode(self):
        """
        Launch a tab as an add with coef operation
        """
        coef, ok = QtGui.QInputDialog.getDouble(self.ui, u'Enter scaling coefficient x (A+xB):', u'Enter coefficient')

        if coef and ok:
            operation = lambda m: m[0] + coef * np.sum(m[1:], 0)
            self.launchmultimode(operation, 'Addition with coef (x=' + coef + ')')

    def subtractwithcoefmode(self):
        """
        Launch a tab as a sub with coef operation
        """
        coef, ok = QtGui.QInputDialog.getDouble(None, u'Enter scaling coefficient x (A-xB):', u'Enter coefficient')

        if coef and ok:
            operation = lambda m: m[0] - coef * np.sum(m[1:], 0)
            self.launchmultimode(operation, 'Subtraction with coef (x=' + str(coef))

    def dividemode(self):
        """
        Launch a tab as a div operation
        """
        operation = lambda m: m[0] / m[1]
        self.launchmultimode(operation, 'Division')

    def averagemode(self):
        """
        Launch a tab as an avg operation
        """
        operation = lambda m: np.mean(m, 0)
        self.launchmultimode(operation, 'Average')

    def launchmultimode(self, operation, operationname):
        """
        Launch a tab in multi-image operation mode
        """
        self.openSelected(operation, operationname)

    def tabCloseRequested(self, index):
        self.centerwidget.widget(index).deleteLater()

    def getCurrentTab(self):
        if self.centerwidget.currentWidget() is None:
            return None
        return self.centerwidget.currentWidget().widget

    def calibrate(self):
        self.getCurrentTab().calibrate()

    def centerfind(self):
        self.getCurrentTab().centerfind()

    def refinecenter(self):
        self.getCurrentTab().refinecenter()

    def redrawcurrent(self):
        try:
            self.getCurrentTab().redrawimage()
        except AttributeError:
            print "Using hack to bypass strange qsignal behavior. Fix this!"

    def remeshmode(self):
        self.getCurrentTab().redrawimage()
        self.getCurrentTab().replot()

    def linecut(self):
        self.getCurrentTab().linecut()

    def vertcut(self):
        self.getCurrentTab().verticalcut()

    def horzcut(self):
        self.getCurrentTab().horizontalcut()

    def roi(self):
        self.getCurrentTab().roi()

    def arccut(self):
        self.getCurrentTab().arccut()

    def polymask(self):
        self.getCurrentTab().polymask()


    def currentChanged(self, index):
        for tab in [self.centerwidget.widget(i) for i in range(self.centerwidget.count())]:
            tab.unload()
        self.centerwidget.currentWidget().load()


    def openfiles(self, paths=None, operation=None, operationname=None):
        self.activate()
        if type(paths) is not list:
            paths = [paths]

        widget = widgets.OOMTabItem(itemclass=widgets.xasviewer, paths=paths)
        self.centerwidget.addTab(widget, os.path.basename(paths[0]))
        self.centerwidget.setCurrentWidget(widget)

    def opendata(self, data=None, operation=None, operationname=None):
        self.activate()
        dimg = loader.diffimage(data=data)
        widget = widgets.OOMTabItem(itemclass=widgets.dimgViewer, dimg=dimg, operation=operation,
                                    operationname=operationname, plotwidget=self.bottomwidget,
                                    toolbar=self.toolbar)
        self.centerwidget.addTab(widget, 'Imported data')
        self.centerwidget.setCurrentWidget(widget)

    def currentImage(self):
        return self.getCurrentTab()

    def replotcurrent(self):
        self.getCurrentTab().replot()

    def invalidatecache(self):
        self.getCurrentTab().dimg.invalidatecache()

    def togglespacegroup(self):
        if self.toolbar.actionSpaceGroup.isChecked():
            self.placeholders[1].setCurrentWidget(self.spacegroupwidget)
        else:
            self.placeholders[1].setCurrentWidget(self.rightwidget)

    def exportimage(self):
        fabimg = edfimage.edfimage(np.rot90(self.getCurrentTab().imageitem.image))
        dialog = QtGui.QFileDialog(parent=None, caption=u"Export image as EDF",
                                   directory=unicode(os.path.dirname(self.getCurrentTab().paths[0])),
                                   filter=u"EDF (*.edf)")
        dialog.selectFile(unicode(os.path.dirname(self.getCurrentTab().paths[0])))
        filename, ok = dialog.getSaveFileName()
        if ok and filename:
            fabimg.write(filename)

    def capture(self):
        self.getCurrentTab().capture()


import os
import RmcView
import time
import subprocess
from PySide import QtGui, QtCore
from pipeline import hig, loader

class gui():
    def __init__(self, ui):
        self.ui = ui

        # self.ui.rmcbutton.clicked.connect(self.showrmc)

        self.ui.rmcaddfiles.clicked.connect(self.addfiles)
        self.ui.rmcremovefiles.clicked.connect(self.removefiles)
        self.ui.rmcExecute.clicked.connect(self.execute)
        self.ui.rmcAddloadingfactor.clicked.connect(self.addloadingfactor)
        self.ui.rmcSubtractloadingfactor.clicked.connect(self.subtractloadingfactor)
        self.ui.rmcopen.clicked.connect(self.open)
        self.ui.rmcreset.clicked.connect(self.reset)
        self.ui.rmcreset_2.clicked.connect(self.reset2)
        # self.ui.rmcRunRemotely.stateChanged.connect(self.runRemotely)

    def open(self):
        Outputdirectory = QtGui.QFileDialog.getExistingDirectory(self.ui, "Select an Output directory")
        self.ui.rmcoutput.setText(Outputdirectory)

    def reset(self):
        self.ui.rmcoutput.setText("")

    def reset2(self):
        self.ui.rmcoutput.setText("")
        self.ui.rmcLoadingfactors.clear()
        self.ui.rmcSteps.setValue(99)
        self.ui.rmcScalefactor.setValue(1)
        self.ui.rmcModlestartsize.setValue(1)
        self.ui.rmcRunName.setText("")
        self.ui.rmcinputpaths.clear()

    def showrmc(self):
        """
        switch to timeline view
        """
        self.ui.viewmode.setCurrentIndex(3)
        self.ui.sidemode.setCurrentIndex(1)

    def addfiles(self):
        paths, ok = QtGui.QFileDialog.getOpenFileNames(self.ui, 'Add files to RMC', os.curdir,
                                                       "*.tif *.edf *.fits *.tif")
        self.ui.rmcinputpaths.addItems(paths)

    def removefiles(self):
        for index in self.ui.rmcinputpaths.selectedIndexes():
            item = self.ui.rmcinputpaths.takeItem(index.row())
            item = None

    def addloadingfactor(self):
        loadingfactor, _ = QtGui.QInputDialog.getDouble(self.ui, "Loading Factor", "Enter a loading factor:", value=0,
                                                        minValue=-0, maxValue=1000, decimals=3)
        newItem = QtGui.QListWidgetItem()
        newItem.setText(str(loadingfactor))
        self.ui.rmcLoadingfactors.addItem(newItem)

    def subtractloadingfactor(self):
        for index in self.ui.rmcLoadingfactors.selectedIndexes():
            item = self.ui.rmcLoadingfactors.takeItem(index.row())
            item = None

    def runRemotely(self):

        ["scp test_input.hig ablair@parratt.lbl.gov:~/  "
            , "ssh -t ablair@parratt.lbl.gov "" "" "]

        RemoteProcess = subprocess.Popen("scp test_input.hig ablair@parratt.lbl.gov:~/  ")
        # print RemoteProcess


    def execute(self):
        steps = self.ui.rmcSteps.value()
        scalefactor = self.ui.rmcScalefactor.value()
        modlestartsize = self.ui.rmcModlestartsize.value()

        loadingfactors = []

        for item in iterAllItems(self.ui.rmcLoadingfactors):
            loadingfactors.append(item.text())

        rip = self.ui.rmcinputpaths
        inputpaths = [rip.item(index).text() for index in xrange(rip.count())]

        tiles = len(loadingfactors)

        for path in inputpaths:
            d = {'hipRMCInput': {'instrumentation': {'inputimage': path,
                                                     'imagesize': loader.loadimage(path).shape[0: 2],
                                                     'numtiles': tiles,
                                                     'loadingfactors': loadingfactors,
                                                     # 'maskimage': "data/mask.tif"
                                                     },  # optional
                                 'computation': {
                                 'runname': os.path.join(self.ui.rmcoutput.text(), self.ui.rmcRunName.text()),
                                                 'modelstartsize': [modlestartsize, modlestartsize],
                                                 'numstepsfactor': steps,
                                                 'scalefactor': scalefactor}}}
            h = hig.hig(**d)
            h.write("test_input.hig")
            self.rmcdaemon = RMCThread()
            self.rmcdaemon.sig_finished.connect(self.displayoutput)
            self.rmcdaemon.start()
            # test_call = 1
            # if test_call == 1:
            #     self.execute()  # Trying to get the window to delete after a new window is opened.

    # while executeNumber < 1:
    # executeNumber += 1
    #     if executeNumber >= 2:
    #         break

    def displayoutput(self, exitcode):
        def checkforresults(self):
            os.path.exists(self.ui.rmcoutput.text())

        def checkforresults2(self):
            if os.listdir(self.ui.rmcoutput.text()) == []:
                return False
            else:
                return True

        if checkforresults(self) is True and checkforresults2(self) is True:

            print "Finished", exitcode

            path = os.path.join(self.ui.rmcoutput.text(), self.ui.rmcRunName.text())

            loadingfactors = []

            for item in iterAllItems(self.ui.rmcLoadingfactors):
                loadingfactors.append(item.text())

            layout = self.ui.rmclayout
            layout.addWidget(RmcView.rmcView(path, loadingfactors))


def iterAllItems(w):
    for i in range(w.count()):
        yield w.item(i)


class RMCThread(QtCore.QThread):
    sig_finished = QtCore.Signal(int)
    def run(self):
        process = subprocess.Popen(['./hiprmc', 'test_input.hig'])
        while process.poll() is None:
            time.sleep(0.5)
        self.sig_finished.emit(process.poll())

            # os.system("./hiprmc test_input.hig")

    def stop(self):
        self.exiting = True
        print ("thread stop - %s" % self.exiting)

    def __del__(self):
        self.exiting = True
        self.wait()



from PySide import QtGui, QtCore  # (the example applies equally well to PySide)
import pyqtgraph as pg
import numpy as np

app = QtGui.QApplication([])
w = QtGui.QWidget()
plot = pg.PlotWidget()

imageData = np.load('waterfall(1).npy')
TID = np.transpose(imageData)  # TID = transposes imageData is creating TID

slider = QtGui.QSlider(QtCore.Qt.Vertical, w)
slider.setGeometry(500, 500, 30, 10)
slider.setFocusPolicy(QtCore.Qt.NoFocus)

curves = []

for i in range(1, TID.shape[1]):
    curves.append(plot.plot(TID[:,i], pen=(i,TID.shape[1]*1.3)))

def getValue(value):
    print value
    for i in range(1, TID.shape[1]):
        curves[i].setData(TID[:,i]+(i*value))
        print i

w.connect(slider, QtCore.SIGNAL('valueChanged(int)'), getValue)

layout = QtGui.QGridLayout()
w.setLayout(layout)

layout.addWidget(plot, 0, 2, 2, 1)
layout.addWidget(slider, 0, 1, 2, 1)

w.show()
app.exec_()
